#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <signal.h>
#include <sys/wait.h>
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif
#include "chassis.p.h"

typedef struct _node_info Node_Info;

void new_node_type(Node *nd);
gint poll_timeout(gpointer data);
GtkWidget *create_pixmap(GtkWidget *w, char *file);
GtkWidget *create_base(void);
void add_pixmap_directory(char *dir);

struct _node_info
{
   int on;
   int overload;
   int power_fault;
   int power_control_fault;
   int intrustion_active;
   int drive_fault;
   int cooling_fan_fault;

   int selected;
   int temp_alert;
   int identify;
   
   int va1k;
   int baytech;
   int emp;
   int vasenet;
   int sbt2;
   int quanta;
   int unknown;
   
   int noo;
   
   int error;
};

GtkWidget *base = NULL;

void
flim_client_message(Msg *msg)
{
   if (!strcmp(msg->msg, "nexxus_add"))
     {
	Nexxus *nx;
	char nx_name[4096], nx_address[4096], nx_login[4096], nx_password[4096];
	
	nx = nexxus_new();
	sscanf(msg->dat, "%[^:]:%[^:]:%[^:]:%[^:]", nx_name, nx_address, nx_login, nx_password);
	nx->name = g_strdup(nx_name);
	nx->address = g_strdup(nx_address);
	nx->login = g_strdup(nx_login);
	nx->password = g_strdup(nx_password);
	nexxus_connect(nx);
	
	  {
	     GtkWidget *vbox;
	     
	     vbox = gtk_object_get_data(GTK_OBJECT(base), "vbox1");
	     if (vbox)
	       {
		  GtkWidget *frame, *vbox2;
		  
		  frame = gtk_frame_new (nx->name);
		  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 2);
		  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
		  gtk_widget_show(frame);
		  gtk_object_set_data(GTK_OBJECT(base), nx->name, frame);
		  
		  vbox2 = gtk_vbox_new (FALSE, 0);
		  gtk_widget_show (vbox2);		  
		  gtk_container_add (GTK_CONTAINER (frame), vbox2);
		  gtk_object_set_data(GTK_OBJECT (frame), "vbox", vbox2);
	       }
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_del"))
     {
	Nexxus *nx;
	
	nx = nexxus_find(msg->dat);
	if (nx) 
	   {
	     GtkWidget *frame;
	     
	     frame = gtk_object_get_data(GTK_OBJECT(base), nx->name);
	     if (frame)
		gtk_widget_destroy(frame);
	      nexxus_free(nx);
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_rename"))
     {
	Nexxus *nx;
	char nx_name[4096], nx_name2[4096];
	
	sscanf(msg->dat, "%[^:]:%[^:]", nx_name, nx_name2);
	nx = nexxus_find(nx_name);
	if (nx)
	  {
	     GtkWidget *frame;
	     
	     if (nx->name) g_free(nx->name);
	     nx->name = g_strdup(nx_name2);
	     frame = gtk_object_get_data(GTK_OBJECT(base), nx_name);
	     if (frame)
	       {
		  gtk_object_remove_data(GTK_OBJECT(base), nx_name);
		  gtk_object_set_data(GTK_OBJECT(base), nx->name, frame);
		  gtk_frame_set_label(GTK_FRAME(frame), nx->name);
	       }
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_address"))
     {
	Nexxus *nx;
	char nx_name[4096], nx_addr[4096], nx_login[4096], nx_pass[4096];

	sscanf(msg->dat, "%[^:]:%[^:]:%[^:]:%[^:]", nx_name, nx_addr, nx_login, nx_pass);
	nx = nexxus_find(nx_name);
	if (nx)
	  {
	     if (nx->address) g_free(nx->address);
	     nx->address = g_strdup(nx_addr);
	     if (nx->login) g_free(nx->login);
	     nx->login = g_strdup(nx_login);
	     if (nx->password) g_free(nx->password);
	     nx->password = g_strdup(nx_pass);
	     if (nx->connection) flim_comms_disconnect_nexxus(nx->connection);
	     nx->connection = NULL;
	     nexxus_connect(nx);
	  }
     }
   else if (!strcmp(msg->msg, "node_add"))
     {
	Nexxus *nx;
	Node *nd;
	Node_Info *ni;
	char nx_name[4096], nd_name[4096];
	
	sscanf(msg->dat, "%[^:]:%[^:]", nx_name, nd_name);
	nx = nexxus_find(nx_name);
	if (nx)
	  {
	     nd = node_new(nx);
	     nd->name = g_strdup(nd_name);
	     ni = g_malloc(sizeof(Node_Info));
	     ni->on = 0;
	     ni->overload = 0;
	     ni->power_fault = 0;
	     ni->power_control_fault = 0;
	     ni->intrustion_active = 0;
	     ni->drive_fault = 0;
	     ni->cooling_fan_fault = 0;
	     ni->selected = 0;
	     ni->temp_alert = 0;
	     ni->identify = 0;
	     ni->va1k = 0;
	     ni->baytech = 0;
	     ni->unknown = 0;
	     ni->vasenet = 0;
	     ni->sbt2 = 0;
	     ni->quanta = 0;
	     ni->emp = 0;
	     ni->error = 0;
	     ni->noo = 1;
	     nd->data = ni;
	  }
	if (nx)
	  {
	     GtkWidget *frame;
	     
	     frame = gtk_object_get_data(GTK_OBJECT(base), nx->name);
	     if (frame)
	       {
		  GtkWidget *vbox;
		  
		  vbox = gtk_object_get_data(GTK_OBJECT(frame), "vbox");
		  if (vbox)
		    {
		       GtkWidget *hbox;

		       frame = gtk_frame_new(nd->name);
		       gtk_widget_show(frame);
		       gtk_container_set_border_width (GTK_CONTAINER (frame), 2);
		       gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
		       gtk_object_set_data(GTK_OBJECT(vbox), nd->name, frame);
		  
		       hbox = gtk_hbox_new (FALSE, 0);
		       gtk_widget_show (hbox);	  
		       gtk_container_add (GTK_CONTAINER (frame), hbox);
		       gtk_object_set_data(GTK_OBJECT (frame), "hbox", hbox);

		    }
	       }
	  }
     }
   else if (!strcmp(msg->msg, "node_del"))
     {
	Node *nd;
	GtkWidget *w, *vbox;
	
	nd = node_find(msg->dat);
	if (nd)
	  {
	     Node_Info *ni;
	     
	     ni = nd->data;
	     g_free(ni);
	     w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
	     vbox = gtk_object_get_data(GTK_OBJECT(w), "vbox");
	     w = gtk_object_get_data(GTK_OBJECT(vbox), nd->name);
	     gtk_widget_destroy(w);
	     node_free(nd);
	  }
     }
   else if (!strcmp(msg->msg, "node_rename"))
     {
	GtkWidget *w, *vbox;
	Node *nd;
        char nx_name[4096], nd_name[4096], nd_name2[4096], buf[4096];
	
	sscanf(msg->dat, "%[^:]:%[^:]:%[^:]", nx_name, nd_name, nd_name2);
	g_snprintf(buf, sizeof(buf), "%s:%s", nx_name, nd_name);
	nd = node_find(buf);
	if (nd)
	  {
	     if (nd->name) g_free(nd->name);
	     nd->name = g_strdup(nd_name2);
	     w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
	     vbox = gtk_object_get_data(GTK_OBJECT(w), "vbox");
	     w = gtk_object_get_data(GTK_OBJECT(vbox), nd_name);
	     gtk_object_remove_data(GTK_OBJECT(base), nd_name);
	     gtk_object_set_data(GTK_OBJECT(vbox), nd->name, w);
	     gtk_frame_set_label(GTK_FRAME(w), nd->name);
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_select"))
     {
	Nexxus *nx;

	nx = nexxus_find(msg->dat);
	if (nx) 
	   {
	      nx->selected = 1;
	   }
     }
   else if (!strcmp(msg->msg, "node_select"))
     {
	Node *nd;

	nd = node_find(msg->dat);
	if (nd) 
	  {
	     nd->selected = 1;
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_unselect"))
     {
	Nexxus *nx;

	nx = nexxus_find(msg->dat);
	if (nx) 
	  {
	     nx->selected = 0;
	  }
     }
   else if (!strcmp(msg->msg, "node_unselect"))
     {
	Node *nd;

	nd = node_find(msg->dat);
	if (nd) 
	  {
	     nd->selected = 0;
	  }
     }
}

void
new_node_type(Node *nd)
{
   Nexxus *nx;
   Node_Info *in;
   
   nx = nd->nexxus;
   in = nd->data;
   
   /* figure out the node type */
     {
	gchar buf[4096];
	
	if (!in->emp)
	  {
	     g_snprintf(buf, sizeof(buf), "EMP:NODE_STATUS:%s", nd->name);
	     FLIM_COMMS_BEGIN(nx->connection, buf);
	     if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
	       {
		  while (FLIM_COMMS_REPLY)
		    {
		       if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			 {
			    in->emp = 1;
			    break;
			 }
		       if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			  break;
		       if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			  break;
		       FILM_COMMS_GET_REPLY;
		    }
	       }
	     else
		in->emp = 0;
	     FILM_COMMS_REPLY_END;
	  }
	if (!in->emp)
	  {
	     g_snprintf(buf, sizeof(buf), "va1000:chassis_status:%s", nd->name);
	     FLIM_COMMS_BEGIN(nx->connection, buf);
	     if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
	       {
		  while (FLIM_COMMS_REPLY)
		    {
		       if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			 {
			    in->va1k = 1;
			    break;
			 }
		       if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			  break;
		       if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			 break;
		       FILM_COMMS_GET_REPLY;
		    }
	       }
	     else
	       in->va1k = 0;
	     FILM_COMMS_REPLY_END;
	     if (!in->va1k)
	       {
		  g_snprintf(buf, sizeof(buf), "baytech:chassis_status:%s", nd->name);
		  FLIM_COMMS_BEGIN(nx->connection, buf);
		  if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
		    {
		       while (FLIM_COMMS_REPLY)
			 {
			    if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			      {
				 in->baytech = 1;
				 break;
			      }
			    if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			       break;
			    FILM_COMMS_GET_REPLY;
			 }
		    }
		  else
		     in->baytech = 0;
		  FILM_COMMS_REPLY_END;
		  if (!in->baytech)
		    {
		       g_snprintf(buf, sizeof(buf), "vasenet:chassis_status:%s", nd->name);
		       FLIM_COMMS_BEGIN(nx->connection, buf);
		       while (FLIM_COMMS_REPLY)
			 {
			    if (FLIM_COMMS_REPLY_COUNT >= 3)
			      {
				 if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
				   {
				      in->vasenet = 1;
				      break;
				   }
				 if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
				   break;
				 if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
				   break;
			      }
			    FILM_COMMS_GET_REPLY;
			 }
		       FILM_COMMS_REPLY_END;
		       if (!in->vasenet)
			 {
			    g_snprintf(buf, sizeof(buf), "sbt2:chassis_status:%s", nd->name);
			    FLIM_COMMS_BEGIN(nx->connection, buf);
			    if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
			      {
				 while (FLIM_COMMS_REPLY)
				   {
				      if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
					{
					   in->sbt2 = 1;
					   break;
					}
				      if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
					break;
				      if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
					break;
				      FILM_COMMS_GET_REPLY;
				   }
			      }
			    else
			      in->sbt2 = 0;
			    FILM_COMMS_REPLY_END;
			    if (!in->sbt2)
			      {
				 g_snprintf(buf, sizeof(buf), "quanta:chassis_status:%s", nd->name);
				 FLIM_COMMS_BEGIN(nx->connection, buf);
				 if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
				   {
				      while (FLIM_COMMS_REPLY)
					{
					   if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
					     {
						in->quanta = 1;
						break;
					     }
					   if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
					     break;
					   if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
					     break;
					   FILM_COMMS_GET_REPLY;
					}
				   }
				 else
				   in->quanta = 0;
				 FILM_COMMS_REPLY_END;
				 if (!in->quanta)
				   in->unknown = 1;
			      }
			 }
		    }
	       }
	  }
     }
}

#define ITEM(tip, name, icon) \
button = gtk_button_new(); \
gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0); \
gtk_widget_show(button); \
gtk_tooltips_set_tip(GTK_TOOLTIPS(tt), button, tip, ""); \
pixmap = create_pixmap(base, icon); \
gtk_widget_ref(pixmap); \
gtk_widget_show(pixmap); \
gtk_container_add(GTK_CONTAINER(button), pixmap); \
gtk_object_set_data(GTK_OBJECT (frame), name, pixmap); \
gtk_widget_set_sensitive(pixmap, 0);

#define SENSOR(r_field, var, w_name) \
p = (var); \
if (!strcmp(FLIM_COMMS_REPLY[r_field], "YES")) (var) = 1; \
else (var) = 0; \
if ((var) != p) { \
w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name); \
w = gtk_object_get_data(GTK_OBJECT(w), "vbox"); \
w = gtk_object_get_data(GTK_OBJECT(w), nd->name); \
w = gtk_object_get_data(GTK_OBJECT(w), w_name); \
if (var) gtk_widget_set_sensitive(w, 1); \
else gtk_widget_set_sensitive(w, 0); \
}

#define VSENSOR(r_field, ch, var, w_name) \
p = (var); \
if (!strcmp(FLIM_COMMS_REPLY[r_field], ch)) (var) = 1; \
else (var) = 0; \
if ((var) != p) { \
w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name); \
w = gtk_object_get_data(GTK_OBJECT(w), "vbox"); \
w = gtk_object_get_data(GTK_OBJECT(w), nd->name); \
w = gtk_object_get_data(GTK_OBJECT(w), w_name); \
if (var) gtk_widget_set_sensitive(w, 1); \
else gtk_widget_set_sensitive(w, 0); \
}

gint
poll_timeout(gpointer data)
{
   GList *l;
   
   flim_client_block_messages();
   for (l = nexxus_list; l; l = l->next)
     {
	Nexxus *nx;
	GList *ll;
	
	nx = l->data;
	if (nx->connection)
	  {
	     for (ll = nx->nodes; ll; ll = ll->next)
	       {
		  Node *nd;
		  Node_Info *in;
		  
		  nd = ll->data;
		  in = nd->data;
		  /* noo */
		  if (in->noo)
		    {
		       GtkWidget *hbox, *button, *pixmap, *frame;
		       GtkTooltips *tt;
		       
		       /* figure out the node type */
		       new_node_type(nd);
		       
		       hbox = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
		       hbox = gtk_object_get_data(GTK_OBJECT(hbox), "vbox");
		       frame = gtk_object_get_data(GTK_OBJECT(hbox), nd->name);
		       hbox = gtk_object_get_data(GTK_OBJECT(frame), "hbox");
		       
		       tt = gtk_tooltips_new();
		       gtk_tooltips_set_delay(GTK_TOOLTIPS(tt), 100);
		       gtk_tooltips_enable(GTK_TOOLTIPS(tt));
		       
		       if (in->va1k)
			 {
			    ITEM(_("Lit if power for node is on"), "pixmap1", "chassis_power.xpm");
			    ITEM(_("Lit if node is selected"), "pixmap2", "selected.xpm");
			    ITEM(_("Lit if temperature alert is active"), "pixmap3", "temp.xpm");
			    ITEM(_("Lit if machine is identifying"), "pixmap4", "identify.xpm");
			 }
		       else if (in->vasenet)
			 {
			    ITEM(_("Lit if power for node is on"), "pixmap1", "chassis_power.xpm");
			    ITEM(_("Lit if node is o in alarm status"), "pixmap2", "selected.xpm");
			    ITEM(_("Lit if nic is active"), "pixmap3", "temp.xpm");
			    ITEM(_("Lit if hard drive is faulty"), "pixmap4", "chassis_drive_fault.xpm");
			 }
		       else if (in->sbt2)
			 {
			    gtk_widget_destroy(hbox);
			    button = gtk_label_new(_("Power State..."));
			    gtk_container_add(GTK_CONTAINER(frame), button);
			    gtk_widget_show(button);
			    gtk_object_set_data(GTK_OBJECT (frame), "label", button);
			 }
		       else if (in->quanta)
			 {
			    gtk_widget_destroy(hbox);
			    button = gtk_label_new(_("SBT2 Chipset. No info"));
			    gtk_container_add(GTK_CONTAINER(frame), button);
			    gtk_widget_show(button);
			 }
		       else if (in->baytech)
			 {
			    ITEM(_("Lit if power to baytech is on"), "pixmap1", "chassis_power.xpm");
			 }
		       else if (in->emp)
			 {
			    ITEM(_("Lit if power for node is on"), "pixmap1", "chassis_power.xpm");
			    ITEM(_("Lit if power for node is overloading"), "pixmap2", "chassis_overload.xpm");
			    ITEM(_("Lit if power fault for node"), "pixmap3", "chassis_power_fault.xpm");
			    ITEM(_("Lit if power control is faulty"), "pixmap4", "chassis_power_control_fault.xpm");
			    ITEM(_("Lit if intrusion sensors have been tripped"), "pixmap5", "chassis_intrusion_active.xpm");
			    ITEM(_("Lit if drive is faulty"), "pixmap6", "chassis_drive_fault.xpm");
			    ITEM(_("Lit if cooling fan is faulty"), "pixmap7", "chassis_cooling_fan_fault.xpm");
			 }
		       else
			 {
			    gtk_widget_destroy(hbox);
			    button = gtk_label_new(_("Unknown node type"));
			    gtk_container_add(GTK_CONTAINER(frame), button);
			    gtk_widget_show(button);
			 }
		       /* no longer new */
		       in->noo = 0;
		    }
		  /* end noo */
		    {
		       char buf[4096];
		       
		       in->error = 0;
		       if (in->emp)
			 {
			    g_snprintf(buf, sizeof(buf), "emp:chassis_status:%s", nd->name);
			    FLIM_COMMS_BEGIN(nx->connection, buf);
			    if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
			      {
				 while (FLIM_COMMS_REPLY)
				   {
				      int p;
				      GtkWidget *w;
				      
				      if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
					 break;
				      if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
					 break;
				      if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
					{
					   in->error = 1;
					   break;
					}
				      if (FLIM_COMMS_REPLY_COUNT >= 18)
					{
					   SENSOR(3, in->on, "pixmap1");
					   SENSOR(4, in->overload, "pixmap2");
					   SENSOR(6, in->power_fault, "pixmap3");
					   SENSOR(7, in->power_control_fault, "pixmap4");
					   SENSOR(14, in->intrustion_active, "pixmap5");
					   SENSOR(16, in->drive_fault, "pixmap6");
					   SENSOR(17, in->cooling_fan_fault, "pixmap7");
					}
				      FILM_COMMS_GET_REPLY;
				   }
			      }
			    FILM_COMMS_REPLY_END;
			 }
		       else if (in->va1k)
			 {
			    g_snprintf(buf, sizeof(buf), "va1000:chassis_status:%s", nd->name);
			    FLIM_COMMS_BEGIN(nx->connection, buf);
			    if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
			      {
				 while (FLIM_COMMS_REPLY)
				   {
				      int p;
				      GtkWidget *w;
				      
				      if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
					 break;
				      if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
					 break;
				      if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
					{
					   in->error = 1;
					   break;
					}
				      if (FLIM_COMMS_REPLY_COUNT >= 9)
					{
					   SENSOR(3, in->on, "pixmap1");
					   SENSOR(4, in->selected, "pixmap2");
					   SENSOR(7, in->temp_alert, "pixmap3");
					   SENSOR(8, in->identify, "pixmap4");
					}
				      FILM_COMMS_GET_REPLY;
				   }
			      }
			    FILM_COMMS_REPLY_END;
			 }
		       else if (in->vasenet)
			 {
			    g_snprintf(buf, sizeof(buf), "vasenet:chassis_status:%s", nd->name);
			    FLIM_COMMS_BEGIN(nx->connection, buf);
			    if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
			      {
				 while (FLIM_COMMS_REPLY)
				   {
				      int p;
				      GtkWidget *w;
				      
				      if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
					 break;
				      if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
					 break;
				      if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
					{
					   in->error = 1;
					   break;
					}
				      if (FLIM_COMMS_REPLY_COUNT >= 6)
					{
					   VSENSOR(2, "P", in->on, "pixmap1");
					   VSENSOR(3, "A", in->selected, "pixmap2");
					   VSENSOR(4, "N", in->temp_alert, "pixmap3");
					   VSENSOR(5, "H", in->identify, "pixmap4");
					}
				      FILM_COMMS_GET_REPLY;
				   }
			      }
			    FILM_COMMS_REPLY_END;
			 }
		       else if (in->sbt2)
			 {
			    g_snprintf(buf, sizeof(buf), "sbt2:chassis_status:%s", nd->name);
			    FLIM_COMMS_BEGIN(nx->connection, buf);
			    if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
			      {
				 while (FLIM_COMMS_REPLY)
				   {
				      int p;
				      GtkWidget *w;
				      
				      if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
					 break;
				      if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
					 break;
				      if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
					{
					   in->error = 1;
					   break;
					}
				      if (FLIM_COMMS_REPLY_COUNT >= 4)
					{
					   w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
					   w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
					   w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
					   w = gtk_object_get_data(GTK_OBJECT (w), "label");
					   gtk_label_set_text(GTK_LABEL(w), FLIM_COMMS_REPLY[3]);
					}
				      FILM_COMMS_GET_REPLY;
				   }
			      }
			    FILM_COMMS_REPLY_END;
			 }
		       else if (in->quanta)
			 {
			 }
			 {
			    GtkWidget *w;
			    
			    w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
			    w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
			    w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
			    if (in->error) gtk_widget_set_sensitive(w, 0);
			    else gtk_widget_set_sensitive(w, 1);
			 }
		    }	     
	       }
	  }
     }   
   
   data = NULL;
   gtk_timeout_add(2000, poll_timeout, NULL);
   flim_client_unblock_messages();
   return 0;
}

GtkWidget *
create_base(void)
{
   GtkWidget *base;
   GtkWidget *vbox1;
   GtkWidget *entry;
   
   base = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_object_set_data (GTK_OBJECT (base), "base", base);
   gtk_container_set_border_width (GTK_CONTAINER (base), 2);
   gtk_window_set_title (GTK_WINDOW (base), "Flim");
   gtk_window_set_wmclass (GTK_WINDOW (base), "Chassis", "FLIM");
   gtk_window_set_policy (GTK_WINDOW (base), 0, 1, 1);
   
   vbox1 = gtk_vbox_new (FALSE, 0);
   gtk_widget_ref (vbox1);
   gtk_object_set_data(GTK_OBJECT (base), "vbox1", vbox1);
   gtk_widget_show (vbox1);
   gtk_container_add (GTK_CONTAINER (base), vbox1);

   entry = gtk_entry_new();
   gtk_widget_show(entry);
   gtk_widget_set_usize(entry, -1, -1);
   gtk_box_pack_start (GTK_BOX (vbox1), entry, FALSE, FALSE, 2);
   gtk_entry_set_editable(GTK_ENTRY(entry), 0);
   gtk_entry_set_text(GTK_ENTRY(entry), _("Chassis status"));
   
   gtk_signal_connect (GTK_OBJECT (base), "destroy",
		       GTK_SIGNAL_FUNC (exit),
		       NULL);
   
   return base;
}

int
main(int argc, char **argv)
{
   Msg msg;
   
   flim_client_init(argc, argv);
#ifdef ENABLE_NLS
   bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
   textdomain (PACKAGE);
#endif
   
   gtk_set_locale ();
   gtk_init (&argc, &argv);
   
   add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
   add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");
   
   base = create_base();
   gtk_widget_realize(base);

   g_snprintf(msg.msg, sizeof(msg.msg), "window_id");
   g_snprintf(msg.dat, sizeof(msg.dat), "%i", (int)GDK_WINDOW_XWINDOW(base->window));
   flim_client_send(&msg);

   for (;;) { XEvent xev; if (XCheckTypedWindowEvent(GDK_WINDOW_XDISPLAY(base->window), GDK_WINDOW_XWINDOW(base->window), ReparentNotify, &xev)) break; usleep(100000); }
   gtk_widget_show(base);

   g_snprintf(msg.msg, sizeof(msg.msg), "ready");
   flim_client_send(&msg);
   
   flim_comms_do_gui(1);   
   gtk_timeout_add(2000, poll_timeout, NULL);
   
   gtk_main();
   return 0;
}
