#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <signal.h>
#include <sys/wait.h>
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif
#include "chassis.p.h"

typedef struct _node_info Node_Info;
typedef struct _sensor Sensor;
typedef struct _job Job;

void
on_next_clicked                    (GtkButton       *button,
				    gpointer         user_data);
void
on_prev_clicked                    (GtkButton       *button,
				    gpointer         user_data);
void draw_sensor(Sensor *s, GtkWidget *frame);
void add_sensor(GtkWidget *vbox, int num, Sensor *s);
void add_sensor(GtkWidget *vbox, int num, Sensor *s);
gint poll_timeout(gpointer data);
GtkWidget *create_base(void);

struct _node_info
{
   int noo;
   int sensors_read;
   int sensor_num;
   Sensor *sensors;
};

struct _sensor
{
   char *name;
   char *units;
   int digital;
   
   double min;
   double lower_noncrit, lower_nonrec, lower_crit;
   double upper_noncrit, upper_nonrec, upper_crit;
   double max;
   double val;
   char  *on;
   char  *error;
};

struct _job
{
   Sensor *sensor;
   char *jobid;
   GtkWidget *widget;
};

GtkWidget *base = NULL;

void
on_next_clicked                    (GtkButton       *button,
				    gpointer         user_data)
{
   gtk_notebook_next_page(GTK_NOTEBOOK(user_data));
   button = NULL;
   user_data = NULL;
}

void
on_prev_clicked                    (GtkButton       *button,
				    gpointer         user_data)
{
   gtk_notebook_prev_page(GTK_NOTEBOOK(user_data));
   button = NULL;
   user_data = NULL;
}

void
flim_client_message(Msg *msg)
{
   if (!strcmp(msg->msg, "nexxus_add"))
     {
	Nexxus *nx;
	char nx_name[4096], nx_address[4096], nx_login[4096], nx_password[4096];
	
	nx = nexxus_new();
	sscanf(msg->dat, "%[^:]:%[^:]:%[^:]:%[^:]", nx_name, nx_address, nx_login, nx_password);
	nx->name = g_strdup(nx_name);
	nx->address = g_strdup(nx_address);
	nx->login = g_strdup(nx_login);
	nx->password = g_strdup(nx_password);
	nexxus_connect(nx);
	
	  {
	     GtkWidget *vbox;
	     
	     vbox = gtk_object_get_data(GTK_OBJECT(base), "vbox1");
	     if (vbox)
	       {
		  GtkWidget *frame, *vbox2;
		  
		  frame = gtk_frame_new (nx->name);
		  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 2);
		  gtk_widget_show(frame);
		  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
		  gtk_object_set_data(GTK_OBJECT(base), nx->name, frame);
		  
		  vbox2 = gtk_vbox_new (FALSE, 2);
		  gtk_widget_show (vbox2);		  
		  gtk_container_add (GTK_CONTAINER (frame), vbox2);
		  gtk_object_set_data(GTK_OBJECT (frame), "vbox", vbox2);
	       }
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_del"))
     {
	Nexxus *nx;
	
	nx = nexxus_find(msg->dat);
	if (nx) 
	   {
	     GtkWidget *frame;
	     
	     frame = gtk_object_get_data(GTK_OBJECT(base), nx->name);
	     if (frame)
		gtk_widget_destroy(frame);
	      nexxus_free(nx);
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_rename"))
     {
	Nexxus *nx;
	char nx_name[4096], nx_name2[4096];
	
	sscanf(msg->dat, "%[^:]:%[^:]", nx_name, nx_name2);
	nx = nexxus_find(nx_name);
	if (nx)
	  {
	     GtkWidget *frame;
	     
	     if (nx->name) g_free(nx->name);
	     nx->name = g_strdup(nx_name2);
	     frame = gtk_object_get_data(GTK_OBJECT(base), nx_name);
	     if (frame)
	       {
		  gtk_object_remove_data(GTK_OBJECT(base), nx_name);
		  gtk_object_set_data(GTK_OBJECT(base), nx->name, frame);
		  gtk_frame_set_label(GTK_FRAME(frame), nx->name);
	       }
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_address"))
     {
	Nexxus *nx;
	char nx_name[4096], nx_addr[4096], nx_login[4096], nx_pass[4096];

	sscanf(msg->dat, "%[^:]:%[^:]:%[^:]:%[^:]", nx_name, nx_addr, nx_login, nx_pass);
	nx = nexxus_find(nx_name);
	if (nx)
	  {
	     if (nx->address) g_free(nx->address);
	     nx->address = g_strdup(nx_addr);
	     if (nx->login) g_free(nx->login);
	     nx->login = g_strdup(nx_login);
	     if (nx->password) g_free(nx->password);
	     nx->password = g_strdup(nx_pass);
	     if (nx->connection) flim_comms_disconnect_nexxus(nx->connection);
	     nx->connection = NULL;
	     nexxus_connect(nx);
	  }
     }
   else if (!strcmp(msg->msg, "node_add"))
     {
	Nexxus *nx;
	Node *nd;
	char nx_name[4096], nd_name[4096];
	
	sscanf(msg->dat, "%[^:]:%[^:]", nx_name, nd_name);
	nx = nexxus_find(nx_name);
	if (nx)
	  {
	     Node_Info *ni;
	     
	     nd = node_new(nx);
	     nd->name = g_strdup(nd_name);
	     ni = g_malloc(sizeof(Node_Info));
	     ni->noo = 1;
	     ni->sensors_read = 0;
	     ni->sensor_num = 0;
	     ni->sensors = NULL;
	     nd->data = ni;
	  }
	if (nx)
	  {
	     GtkWidget *frame;
	     
	     frame = gtk_object_get_data(GTK_OBJECT(base), nx->name);
	     if (frame)
	       {
		  GtkWidget *vbox;
		  
		  vbox = gtk_object_get_data(GTK_OBJECT(frame), "vbox");
		  if (vbox)
		    {
		       GtkWidget *vbox2, *box, *hbox, *button, *arrow;

		       vbox2 = gtk_notebook_new ();
		       
		       frame = gtk_frame_new(nd->name);
		       gtk_container_set_border_width (GTK_CONTAINER (frame), 2);
		       gtk_widget_show(frame);
		       gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
		       gtk_object_set_data(GTK_OBJECT(vbox), nd->name, frame);
		  
		       box = gtk_hbox_new (FALSE, 1);
		       gtk_widget_show(box);
		       gtk_container_add (GTK_CONTAINER (frame), box);
		       gtk_object_set_data(GTK_OBJECT (frame), "box", box);

		       hbox = gtk_vbox_new (FALSE, 1);
		       gtk_widget_show(hbox);
		       gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 1);
		       
		       button = gtk_button_new();
		       gtk_widget_show(button);
		       gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 1);
		       
		       gtk_signal_connect (GTK_OBJECT (button), "clicked",
					   GTK_SIGNAL_FUNC (on_prev_clicked),
					   vbox2);
		       
		       arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_IN);
		       gtk_widget_show(arrow);
		       gtk_container_add (GTK_CONTAINER (button), arrow);

		       button = gtk_button_new();
		       gtk_widget_show(button);
		       gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 1);

		       gtk_signal_connect (GTK_OBJECT (button), "clicked",
					   GTK_SIGNAL_FUNC (on_next_clicked),
					   vbox2);
		       
		       arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_IN);
		       gtk_widget_show(arrow);
		       gtk_container_add (GTK_CONTAINER (button), arrow);
		       
		       gtk_notebook_set_scrollable(GTK_NOTEBOOK(vbox2), TRUE);
		       gtk_notebook_popup_enable(GTK_NOTEBOOK(vbox2));
		       gtk_widget_show (vbox2);	  
		       gtk_box_pack_start (GTK_BOX (box), vbox2, FALSE, FALSE, 1);
		       gtk_object_set_data(GTK_OBJECT (frame), "vbox2", vbox2);

		    }
	       }
	  }
     }
   else if (!strcmp(msg->msg, "node_del"))
     {
	Node *nd;
	GtkWidget *w, *vbox;

	nd = node_find(msg->dat);
	if (nd)
	  {
	     Node_Info *ni;
	     gint i;
	     
	     ni = nd->data;
	     for (i = 0; i < ni->sensor_num; i++)
	       {
		  if (ni->sensors[i].name) 
		     {
			g_free(ni->sensors[i].name);
			if (ni->sensors[i].units) 
			   g_free(ni->sensors[i].units);
			if (ni->sensors[i].on) 
			   g_free(ni->sensors[i].on);
			if (ni->sensors[i].error) 
			   g_free(ni->sensors[i].error);
		     }
	       }
	     if (ni->sensors) g_free(ni->sensors);
	     g_free(ni);
	     w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
	     vbox = gtk_object_get_data(GTK_OBJECT(w), "vbox");
	     w = gtk_object_get_data(GTK_OBJECT(vbox), nd->name);
	     gtk_widget_destroy(w);
	     node_free(nd);
	  }
     }
   else if (!strcmp(msg->msg, "node_rename"))
     {
	GtkWidget *w, *vbox;
	Node *nd;
        char nx_name[4096], nd_name[4096], nd_name2[4096], buf[4096];
	
	sscanf(msg->dat, "%[^:]:%[^:]:%[^:]", nx_name, nd_name, nd_name2);
	g_snprintf(buf, sizeof(buf), "%s:%s", nx_name, nd_name);
	nd = node_find(buf);
	if (nd)
	  {
	     if (nd->name) g_free(nd->name);
	     nd->name = g_strdup(nd_name2);
	     w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
	     vbox = gtk_object_get_data(GTK_OBJECT(w), "vbox");
	     w = gtk_object_get_data(GTK_OBJECT(vbox), nd_name);
	     gtk_object_remove_data(GTK_OBJECT(base), nd_name);
	     gtk_object_set_data(GTK_OBJECT(vbox), nd->name, w);
	     gtk_frame_set_label(GTK_FRAME(w), nd->name);
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_select"))
     {
	Nexxus *nx;

	nx = nexxus_find(msg->dat);
	if (nx) 
	   {
	      nx->selected = 1;
	   }
     }
   else if (!strcmp(msg->msg, "node_select"))
     {
	Node *nd;

	nd = node_find(msg->dat);
	if (nd) 
	  {
	     nd->selected = 1;
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_unselect"))
     {
	Nexxus *nx;

	nx = nexxus_find(msg->dat);
	if (nx) 
	  {
	     nx->selected = 0;
	  }
     }
   else if (!strcmp(msg->msg, "node_unselect"))
     {
	Node *nd;

	nd = node_find(msg->dat);
	if (nd) 
	  {
	     nd->selected = 0;
	  }
     }
}

void
draw_sensor(Sensor *s, GtkWidget *frame)
{
   GtkWidget *entry, *area;
   gchar buf[256];

   if (!frame) return;
   entry = gtk_object_get_data(GTK_OBJECT(frame), "entry");
   if (s->error)
     {
	g_snprintf(buf, sizeof(buf), "ERROR: %s", s->error);
	gtk_entry_set_text(GTK_ENTRY(entry), buf);
	g_free(s->error);
	s->error = NULL;
     }
   else if (!s->digital)
     {
	g_snprintf(buf, sizeof(buf), "ERROR READING");
	if (s->units)
	   g_snprintf(buf, sizeof(buf), "%3.3f %s", s->val, s->units);
	gtk_entry_set_text(GTK_ENTRY(entry), buf);
   
	area = gtk_object_get_data(GTK_OBJECT(frame), "area");
	gtk_progress_configure(GTK_PROGRESS(area), s->val, s->min, s->max);
     }
   else
     {
	if (s->on)
	   gtk_entry_set_text(GTK_ENTRY(entry), s->on);
	else
	   gtk_entry_set_text(GTK_ENTRY(entry), "ERROR READING");
     }
}

void
add_sensor(GtkWidget *vbox, int num, Sensor *s)
{
   GtkWidget *entry, *frame2, *area, *hbox, *label, *vbox2, *hbox2;
   gchar buf[4096];

   if (s->val < s->min) s->val = s->min;
   if (s->val > s->max) s->val = s->max;
   
   hbox = gtk_hbox_new (FALSE, 2);
   gtk_widget_show(hbox);

   label = gtk_label_new(s->name);
   gtk_widget_show(label);
   gtk_notebook_append_page(GTK_NOTEBOOK(vbox), hbox, label);
   g_snprintf(buf, sizeof(buf), "%i", num);
   gtk_object_set_data(GTK_OBJECT(vbox), buf, hbox);
   
   entry = gtk_entry_new();
   gtk_entry_set_editable(GTK_ENTRY(entry), 0);
   gtk_widget_show(entry);
   gtk_widget_set_usize(entry, 128, -1);
   gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 2);

   gtk_object_set_data(GTK_OBJECT(hbox), "entry", entry);
   
   if (!s->digital)
     {
	double size;
	GdkColor color;
	
	vbox2 = gtk_vbox_new (FALSE, 1);
	gtk_widget_show(vbox2);
	gtk_box_pack_start (GTK_BOX (hbox), vbox2, FALSE, FALSE, 2);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show(hbox2);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, FALSE, 0);

	size = (96 * (s->lower_nonrec - s->min)) / (s->max - s->min);
	if (size >= 1.0)
	  {
	     frame2 = gtk_frame_new(NULL);
	     gtk_widget_show(frame2);
	     gtk_box_pack_start (GTK_BOX (hbox2), frame2, FALSE, FALSE, 0);
	     gtk_widget_set_usize(frame2, size, 6);
	     gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
	     area = gtk_drawing_area_new();
	     gtk_widget_show(area);
	     gtk_container_add(GTK_CONTAINER(frame2), area);

	     gtk_widget_realize(area);
	     color.red = 0xffff; color.green = 0x0000; color.blue = 0x0000;
	     gdk_colormap_alloc_color(gtk_widget_get_colormap(area), &color, FALSE, TRUE);
	     gdk_window_set_background(area->window, &color);
	  }

	size = (96 * (s->lower_crit - s->lower_nonrec)) / (s->max - s->min);
	if (size >= 1.0)
	  {
	     frame2 = gtk_frame_new(NULL);
	     gtk_widget_show(frame2);
	     gtk_box_pack_start (GTK_BOX (hbox2), frame2, FALSE, FALSE, 0);
	     gtk_widget_set_usize(frame2, size, 6);
	     gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
	     area = gtk_drawing_area_new();
	     gtk_widget_show(area);
	     gtk_container_add(GTK_CONTAINER(frame2), area);

	     gtk_widget_realize(area);
	     color.red = 0xffff; color.green = 0x8888; color.blue = 0x0000;
	     gdk_colormap_alloc_color(gtk_widget_get_colormap(area), &color, FALSE, TRUE);
	     gdk_window_set_background(area->window, &color);
	  }

	size = (96 * (s->lower_noncrit - s->lower_crit)) / (s->max - s->min);
	if (size >= 1.0)
	  {
	     frame2 = gtk_frame_new(NULL);
	     gtk_widget_show(frame2);
	     gtk_box_pack_start (GTK_BOX (hbox2), frame2, FALSE, FALSE, 0);
	     gtk_widget_set_usize(frame2, size, 6);
	     gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
	     area = gtk_drawing_area_new();
	     gtk_widget_show(area);
	     gtk_container_add(GTK_CONTAINER(frame2), area);
	     
	     gtk_widget_realize(area);
	     color.red = 0xffff; color.green = 0xffff; color.blue = 0x0000;
	     gdk_colormap_alloc_color(gtk_widget_get_colormap(area), &color, FALSE, TRUE);
	     gdk_window_set_background(area->window, &color);
	  }

	size = (96 * (s->upper_noncrit - s->lower_noncrit)) / (s->max - s->min);
	if (size >= 1.0)
	  {
	     frame2 = gtk_frame_new(NULL);
	     gtk_widget_show(frame2);
	     gtk_box_pack_start (GTK_BOX (hbox2), frame2, FALSE, FALSE, 0);
	     gtk_widget_set_usize(frame2, size, 6);
	     gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
	     area = gtk_drawing_area_new();
	     gtk_widget_show(area);
	     gtk_container_add(GTK_CONTAINER(frame2), area);
	     
	     gtk_widget_realize(area);
	     color.red = 0x0000; color.green = 0xffff; color.blue = 0x0000;
	     gdk_colormap_alloc_color(gtk_widget_get_colormap(area), &color, FALSE, TRUE);
	     gdk_window_set_background(area->window, &color);
	  }
	
	size = (96 * (s->upper_crit - s->upper_noncrit)) / (s->max - s->min);
	if (size >= 1.0)
	  {
	     frame2 = gtk_frame_new(NULL);
	     gtk_widget_show(frame2);
	     gtk_box_pack_start (GTK_BOX (hbox2), frame2, FALSE, FALSE, 0);
	     gtk_widget_set_usize(frame2, size, 6);
	     gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
	     area = gtk_drawing_area_new();
	     gtk_widget_show(area);
	     gtk_container_add(GTK_CONTAINER(frame2), area);
	     
	     gtk_widget_realize(area);
	     color.red = 0xffff; color.green = 0xffff; color.blue = 0x0000;
	     gdk_colormap_alloc_color(gtk_widget_get_colormap(area), &color, FALSE, TRUE);
	     gdk_window_set_background(area->window, &color);
	  }

	size = (96 * (s->upper_nonrec - s->upper_crit)) / (s->max - s->min);
	if (size >= 1.0)
	  {
	     frame2 = gtk_frame_new(NULL);
	     gtk_widget_show(frame2);
	     gtk_box_pack_start (GTK_BOX (hbox2), frame2, FALSE, FALSE, 0);
	     gtk_widget_set_usize(frame2, size, 6);
	     gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
	     area = gtk_drawing_area_new();
	     gtk_widget_show(area);
	     gtk_container_add(GTK_CONTAINER(frame2), area);
	     
	     gtk_widget_realize(area);
	     color.red = 0xffff; color.green = 0x8888; color.blue = 0x0000;
	     gdk_colormap_alloc_color(gtk_widget_get_colormap(area), &color, FALSE, TRUE);
	     gdk_window_set_background(area->window, &color);
	  }

	size = (96 * (s->max - s->upper_nonrec)) / (s->max - s->min);
	if (size >= 1.0)
	  {
	     frame2 = gtk_frame_new(NULL);
	     gtk_widget_show(frame2);
	     gtk_box_pack_start (GTK_BOX (hbox2), frame2, FALSE, FALSE, 0);
	     gtk_widget_set_usize(frame2, size, 6);
	     gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
	     area = gtk_drawing_area_new();
	     gtk_widget_show(area);
	     gtk_container_add(GTK_CONTAINER(frame2), area);
	     gtk_widget_realize(area);
	     
	     color.red = 0xffff; color.green = 0x0000; color.blue = 0x0000;
	     gdk_colormap_alloc_color(gtk_widget_get_colormap(area), &color, FALSE, TRUE);
	     gdk_window_set_background(area->window, &color);
	  }
	
	area = gtk_progress_bar_new();
	gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(area), GTK_PROGRESS_LEFT_TO_RIGHT);
	gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR(area), GTK_PROGRESS_CONTINUOUS);
	if (s->val < s->min) s->val = s->min;
	if (s->val > s->max) s->val = s->max;
	gtk_progress_configure(GTK_PROGRESS(area), s->val, s->min, s->max);
	gtk_widget_show(area);
	gtk_widget_set_usize(area, 96, 16);
	gtk_box_pack_start (GTK_BOX (vbox2), area, FALSE, FALSE, 2);

	gtk_object_set_data(GTK_OBJECT(hbox), "area", area);
     }
}

gint
poll_timeout(gpointer data)
{
   GList *l;
   int i;
   int didnoo = 0;
   
   flim_client_block_messages();
   for (l = nexxus_list; l; l = l->next)
     {
	Nexxus *nx;
	GList *ll;
	
	nx = l->data;
	for (ll = nx->nodes; ll; ll = ll->next)
	  {
	     Node *nd;
	     Node_Info *in;
	     
	     nd = ll->data;
	     in = nd->data;
	     /* new node - figure out if it does sensors */
	     if ((in->noo) && (nx->connection))
	       {
		  char buf[4096];
		  
		  g_snprintf(buf, sizeof(buf), "EMP:GET_SENSOR_LIST:%s", nd->name);
		  FLIM_COMMS_BEGIN(nx->connection, buf);
		  if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
		    {
		       while (FLIM_COMMS_REPLY)
			 {
			    if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			      {
				 break;
			      }
			    if ((FLIM_COMMS_REPLY_COUNT >= 6) &&
				(!strcmp("SENSOR", FLIM_COMMS_REPLY[2])))
			      {
				 if (strcmp(FLIM_COMMS_REPLY[5], "UNSUPPORTED"))
				   {
				      int num;
				      char *name;
				      char *type;
				      
				      num = atoi(FLIM_COMMS_REPLY[3]);
				      name = FLIM_COMMS_REPLY[4];
				      type = FLIM_COMMS_REPLY[5];
				      if (num > in->sensor_num)
					{
					   int i, p;
					   
					   p = in->sensor_num;
					   in->sensor_num = num;
					   in->sensors = realloc(in->sensors, 
								 in->sensor_num * sizeof(Sensor));
					   for (i = p; i < in->sensor_num; i++)
					     {
						in->sensors[i].name = NULL;
					     }
					}
				      num--;
				      in->sensors[num].name = g_strdup(name);
				      in->sensors[num].units = NULL;
				      if (!strcmp(type, "DIGITAL"))
					 in->sensors[num].digital = 1;
				      else
					 in->sensors[num].digital = 0;
				      in->sensors[num].min = 0.0;
				      in->sensors[num].lower_noncrit = 0.0;
				      in->sensors[num].lower_nonrec = 0.0;
				      in->sensors[num].lower_crit = 0.0;
				      in->sensors[num].upper_noncrit = 0.0;
				      in->sensors[num].upper_nonrec = 0.0;
				      in->sensors[num].upper_crit = 0.0;
				      in->sensors[num].max = 0.0;
				      in->sensors[num].val = 0.0;
				      in->sensors[num].on = NULL;
				      in->sensors[num].error = NULL;
				   }
			      }
			    
			    FILM_COMMS_GET_REPLY;
			 }
		    }
		  FILM_COMMS_REPLY_END;
		  
		  for (i = 0; i < in->sensor_num; i++)
		    {
		       if ((in->sensors[i].name) && (!in->sensors[i].digital))
			 {
			    g_snprintf(buf, sizeof(buf), "EMP:GET_SENSOR_THRESHOLDS:%s:%i", nd->name, i + 1);
			    FLIM_COMMS_BEGIN(nx->connection, buf);
			    if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
			      {
				 while (FLIM_COMMS_REPLY)
				   {
				      GtkWidget *w;
				      
				      if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
					 break;
				      if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
					 break;
				      if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
					 break;
				      if ((FLIM_COMMS_REPLY_COUNT >= 12) && 
					  (!strcmp("THRESHOLD", FLIM_COMMS_REPLY[2])))
					{
					   in->sensors_read = 1;
					   in->sensors[i].min = atof(FLIM_COMMS_REPLY[4]);
					   in->sensors[i].lower_noncrit = atof(FLIM_COMMS_REPLY[5]);
					   in->sensors[i].lower_nonrec =  atof(FLIM_COMMS_REPLY[6]);
					   in->sensors[i].lower_crit =  atof(FLIM_COMMS_REPLY[7]);
					   in->sensors[i].upper_noncrit =  atof(FLIM_COMMS_REPLY[8]);
					   in->sensors[i].upper_nonrec =  atof(FLIM_COMMS_REPLY[9]);
					   in->sensors[i].upper_crit =  atof(FLIM_COMMS_REPLY[10]);
					   in->sensors[i].max =  atof(FLIM_COMMS_REPLY[11]);
					   
#define DBL_SWAP(a, b) { double _tmp; _tmp = a; a = b; b = _tmp; }
					   
					   if (in->sensors[i].max < in->sensors[i].min)
					     {
						DBL_SWAP(in->sensors[i].min, in->sensors[i].max);
						DBL_SWAP(in->sensors[i].lower_noncrit, in->sensors[i].upper_noncrit);
						DBL_SWAP(in->sensors[i].lower_crit, in->sensors[i].upper_crit);
						DBL_SWAP(in->sensors[i].lower_nonrec, in->sensors[i].upper_nonrec);
					     }
					   
					   if (in->sensors[i].lower_nonrec == 0.0)
					      in->sensors[i].lower_nonrec = in->sensors[i].min;
					   if (in->sensors[i].lower_crit == 0.0)
					      in->sensors[i].lower_crit = in->sensors[i].lower_nonrec;
					   if (in->sensors[i].lower_noncrit == 0.0)
					      in->sensors[i].lower_noncrit = in->sensors[i].lower_crit;
					   if (in->sensors[i].upper_nonrec == 0.0)
					      in->sensors[i].upper_nonrec = in->sensors[i].max;
					   if (in->sensors[i].upper_crit == 0.0)
					      in->sensors[i].upper_crit = in->sensors[i].upper_nonrec;
					   if (in->sensors[i].upper_noncrit == 0.0)
					      in->sensors[i].upper_noncrit = in->sensors[i].upper_crit;
					   
					   w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
					   w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
					   w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
					   w = gtk_object_get_data(GTK_OBJECT(w), "vbox2");
					   
					   add_sensor(w, i, &(in->sensors[i]));
					}
				      
				      FILM_COMMS_GET_REPLY;
				   }
			      }
			    FILM_COMMS_REPLY_END;
			 }
		       else if ((in->sensors[i].name) && (in->sensors[i].digital))
			 {
			    GtkWidget *w;
			    
			    w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
			    w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
			    w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
			    w = gtk_object_get_data(GTK_OBJECT(w), "vbox2");
			    
			    add_sensor(w, i, &(in->sensors[i]));
			 }
		    }
		  in->noo = 0;
		  if (!in->sensors_read)
		    {
		       GtkWidget *w, *box, *label;
		       
		       w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
		       w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
		       w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
		       box = gtk_object_get_data(GTK_OBJECT(w), "box");
		       gtk_widget_destroy(box);
		       label = gtk_label_new(_("No sensors found"));
		       gtk_widget_show(label);
		       gtk_container_add(GTK_CONTAINER(w), label);
		    }
		  didnoo = 1;
	       }
	  }
     }   
   if (didnoo)
     {
	gtk_widget_hide(base);
	gtk_widget_show(base);
     }
   for (l = nexxus_list; l; l = l->next)
     {
	Nexxus *nx;
	GList *ll;
	GList *jobs;
	
	jobs = NULL;
	nx = l->data;
	for (ll = nx->nodes; ll; ll = ll->next)
	  {
	     Node *nd;
	     Node_Info *in;
	     
	     nd = ll->data;
	     in = nd->data;
	     if ((in->sensors_read) && (nx->connection))
	       {
                  for (i = 0; i < in->sensor_num; i++)
		    {
		       char buf[4096];
		       
		       if (in->sensors[i].name)
			 {
			    int reply_count;
			    char **reply;
			    
			    g_snprintf(buf, sizeof(buf), "EMP:READ_SENSOR:%s:%i", nd->name, i + 1);
			    flim_comms_send_msg(nx->connection, buf);
			    while ((reply = flim_comms_get_reply(nx->connection, &reply_count)))
			      {
				 if ((reply_count == 3) &&
				     (!strcmp(reply[2], "JOB_STARTED")))
				   {
				      Job *j;
				      GtkWidget *w;
				      
				      w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
				      w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
				      w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
				      w = gtk_object_get_data(GTK_OBJECT(w), "vbox2");
				      g_snprintf(buf, sizeof(buf), "%i", i);
				      w = gtk_object_get_data(GTK_OBJECT(w), buf);
				      j = g_malloc(sizeof(Job));
				      j->jobid = g_strdup(reply[1]);
				      j->sensor = &(in->sensors[i]);
				      j->widget = w;
				      jobs = g_list_append(jobs, j);
				      flim_comms_free_reply(reply, reply_count);
				      break;
				   }
				 /* error in job - remove */
				 else if ((reply_count >= 3) &&
					  ((!strcmp(reply[2], "JOB_ERROR")) ||
					   (!strcmp(reply[2], "MODULE_NOT_FOUND")) ||
					   (!strcmp(reply[2], "JOB_COMPLETED"))))
				   {
				      GList *l;
				      
				      for (l = jobs; l; l = l->next)
					{
					   Job *j;
					   
					   j = l->data;
					   if (!strcmp(reply[1], j->jobid))
					     {
						if ((reply_count >= 4) && (!strcmp(reply[2], "JOB_ERROR")))
						  {
						     j->sensor->error = g_strdup(reply[3]);
						     draw_sensor(j->sensor, j->widget);
						  }
						g_free(j->jobid);
						g_free(j);
						jobs = g_list_remove(jobs, j);
						break;
					     }
					}
				   }
				 else if ((reply_count >= 3) &&
					  (!strcmp(reply[2], "SENSOR")))
				   {
				      GList *l;
				      
				      for (l = jobs; l; l = l->next)
					{
					   Job *j;
					   
					   j = l->data;
					   if (!strcmp(reply[1], j->jobid))
					     {
						if ((j->sensor->digital) && (reply_count >= 5))
						  {
						     if (j->sensor->on) g_free(j->sensor->on);
						     j->sensor->on = g_strdup(reply[4]);
						  }
						else
						  {
						     if (reply_count >= 5)
							j->sensor->val = atof(reply[4]);
						     if (reply_count >= 6)
						       {
							  if (j->sensor->units) g_free(j->sensor->units);
							  j->sensor->units = g_strdup(reply[5]);
						       }
						  }
						draw_sensor(j->sensor, j->widget);
						break;
					     }
					}
				   }
				 flim_comms_free_reply(reply, reply_count);
			      }
			 }
		    }
	       }	     
	  }
	while (jobs)
	  {
	     int reply_count;
	     char **reply;
	     
	     reply = flim_comms_get_reply(nx->connection, &reply_count);
	     if ((reply_count >= 3) &&
		 ((!strcmp(reply[2], "JOB_ERROR")) ||
		  (!strcmp(reply[2], "MODULE_NOT_FOUND")) ||
		  (!strcmp(reply[2], "JOB_COMPLETED"))))
	       {
		  GList *l;
		  
		  for (l = jobs; l; l = l->next)
		    {
		       Job *j;
		       
		       j = l->data;
		       if (!strcmp(reply[1], j->jobid))
			 {
			    if ((reply_count >= 4) && (!strcmp(reply[2], "JOB_ERROR")))
			      {
				 j->sensor->error = g_strdup(reply[3]);
				 draw_sensor(j->sensor, j->widget);
			      }
			    g_free(j->jobid);
			    g_free(j);
			    jobs = g_list_remove(jobs, j);
			    break;
			 }
		    }
	       }
	     else if ((reply_count >= 3) &&
		      (!strcmp(reply[2], "SENSOR")))
	       {
		  GList *l;
		  
		  for (l = jobs; l; l = l->next)
		    {
		       Job *j;
		       
		       j = l->data;
		       if (!strcmp(reply[1], j->jobid))
			 {
			    if ((j->sensor->digital) && (reply_count >= 5))
			      {
				 if (j->sensor->on) g_free(j->sensor->on);
				 j->sensor->on = g_strdup(reply[4]);
			      }
			    else
			      {
				 if (reply_count >= 5)
				    j->sensor->val = atof(reply[4]);
				 if (reply_count >= 6)
				   {
				      if (j->sensor->units) g_free(j->sensor->units);
				      j->sensor->units = g_strdup(reply[5]);
				   }
			      }
			    draw_sensor(j->sensor, j->widget);
			    break;
			 }
		    }
	       }
	     flim_comms_free_reply(reply, reply_count);
	  }
     }
   data = NULL;
   gtk_timeout_add(5000, poll_timeout, NULL);
   flim_client_unblock_messages();
   return 0;
}

GtkWidget *
create_base(void)
{
   GtkWidget *base;
   GtkWidget *vbox1;
   GtkWidget *entry;
   
   base = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_object_set_data (GTK_OBJECT (base), "base", base);
   gtk_container_set_border_width (GTK_CONTAINER (base), 2);
   gtk_window_set_title (GTK_WINDOW (base), "Flim");
   gtk_window_set_wmclass (GTK_WINDOW (base), "Sensors", "FLIM");
   gtk_window_set_policy (GTK_WINDOW (base), 0, 1, 1);
   
   vbox1 = gtk_vbox_new (FALSE, 0);
   gtk_widget_ref (vbox1);
   gtk_object_set_data(GTK_OBJECT (base), "vbox1", vbox1);
   gtk_widget_show (vbox1);
   gtk_container_add (GTK_CONTAINER (base), vbox1);

   entry = gtk_entry_new();
   gtk_widget_show(entry);
   gtk_widget_set_usize(entry, -1, -1);
   gtk_box_pack_start (GTK_BOX (vbox1), entry, FALSE, FALSE, 2);
   gtk_entry_set_editable(GTK_ENTRY(entry), 0);
   gtk_entry_set_text(GTK_ENTRY(entry), _("Machine Sensors"));
   
   gtk_signal_connect (GTK_OBJECT (base), "destroy",
		       GTK_SIGNAL_FUNC (exit),
		       NULL);
   
   return base;
}

int
main(int argc, char **argv)
{
   Msg msg;
   
   flim_client_init(argc, argv);
#ifdef ENABLE_NLS
   bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
   textdomain (PACKAGE);
#endif
   
   gtk_set_locale ();
   gtk_init (&argc, &argv);
   
   base = create_base();
   gtk_widget_realize(base);

   g_snprintf(msg.msg, sizeof(msg.msg), "window_id");
   g_snprintf(msg.dat, sizeof(msg.dat), "%i", (int)GDK_WINDOW_XWINDOW(base->window));
   flim_client_send(&msg);

   for (;;) { XEvent xev; if (XCheckTypedWindowEvent(GDK_WINDOW_XDISPLAY(base->window), GDK_WINDOW_XWINDOW(base->window), ReparentNotify, &xev)) break; usleep(100000); }
   gtk_widget_show(base);
   
   g_snprintf(msg.msg, sizeof(msg.msg), "ready");
   flim_client_send(&msg);

   flim_comms_do_gui(1);   
   gtk_timeout_add(5000, poll_timeout, NULL);
   
   gtk_main();
   return 0;
}
