#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <signal.h>
#include <sys/wait.h>
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif
#include "systat.p.h"

typedef struct _node_info Node_Info;
typedef struct _sstats Sstats;

void
on_next_clicked                    (GtkButton       *button,
				    gpointer         user_data);
void
on_prev_clicked                    (GtkButton       *button,
				    gpointer         user_data);
gint poll_timeout(gpointer data);
GtkWidget *create_base(void);

struct _sstats
{
   gfloat cpu_load[128];
   gfloat mem_used[60];
   gfloat swap_used[60];
};

struct _node_info
{
   int noo;
   int dostat;
   Sstats stats;
};

GtkWidget *base = NULL;

void
on_next_clicked                    (GtkButton       *button,
				    gpointer         user_data)
{
   gtk_notebook_next_page(GTK_NOTEBOOK(user_data));
   button = NULL;
   user_data = NULL;
}

void
on_prev_clicked                    (GtkButton       *button,
				    gpointer         user_data)
{
   gtk_notebook_prev_page(GTK_NOTEBOOK(user_data));
   button = NULL;
   user_data = NULL;
}

void
flim_client_message(Msg *msg)
{
   if (!strcmp(msg->msg, "nexxus_add"))
     {
	Nexxus *nx;
	char nx_name[4096], nx_address[4096], nx_login[4096], nx_password[4096];
	
	nx = nexxus_new();
	sscanf(msg->dat, "%[^:]:%[^:]:%[^:]:%[^:]", nx_name, nx_address, nx_login, nx_password);
	nx->name = g_strdup(nx_name);
	nx->address = g_strdup(nx_address);
	nx->login = g_strdup(nx_login);
	nx->password = g_strdup(nx_password);
	nexxus_connect(nx);
	
	  {
	     GtkWidget *vbox;
	     
	     vbox = gtk_object_get_data(GTK_OBJECT(base), "vbox1");
	     if (vbox)
	       {
		  GtkWidget *frame, *vbox2;
		  
		  frame = gtk_frame_new (nx->name);
		  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 2);
		  gtk_widget_show(frame);
		  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
		  gtk_object_set_data(GTK_OBJECT(base), nx->name, frame);
		  
		  vbox2 = gtk_vbox_new (FALSE, 2);
		  gtk_widget_show (vbox2);		  
		  gtk_container_add (GTK_CONTAINER (frame), vbox2);
		  gtk_object_set_data(GTK_OBJECT (frame), "vbox", vbox2);
	       }
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_del"))
     {
	Nexxus *nx;
	
	nx = nexxus_find(msg->dat);
	if (nx) 
	  {
	     GtkWidget *frame;
	     
	     frame = gtk_object_get_data(GTK_OBJECT(base), nx->name);
	     if (frame)
		gtk_widget_destroy(frame);
	     nexxus_free(nx);
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_rename"))
     {
	Nexxus *nx;
	char nx_name[4096], nx_name2[4096];
	
	sscanf(msg->dat, "%[^:]:%[^:]", nx_name, nx_name2);
	nx = nexxus_find(nx_name);
	if (nx)
	  {
	     GtkWidget *frame;
	     
	     if (nx->name) g_free(nx->name);
	     nx->name = g_strdup(nx_name2);
	     frame = gtk_object_get_data(GTK_OBJECT(base), nx_name);
	     if (frame)
	       {
		  gtk_object_remove_data(GTK_OBJECT(base), nx_name);
		  gtk_object_set_data(GTK_OBJECT(base), nx->name, frame);
		  gtk_frame_set_label(GTK_FRAME(frame), nx->name);
	       }
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_address"))
     {
	Nexxus *nx;
	char nx_name[4096], nx_addr[4096], nx_login[4096], nx_pass[4096];
	
	sscanf(msg->dat, "%[^:]:%[^:]:%[^:]:%[^:]", nx_name, nx_addr, nx_login, nx_pass);
	nx = nexxus_find(nx_name);
	if (nx)
	  {
	     if (nx->address) g_free(nx->address);
	     nx->address = g_strdup(nx_addr);
	     if (nx->login) g_free(nx->login);
	     nx->login = g_strdup(nx_login);
	     if (nx->password) g_free(nx->password);
	     nx->password = g_strdup(nx_pass);
	     if (nx->connection) flim_comms_disconnect_nexxus(nx->connection);
	     nx->connection = NULL;
	     nexxus_connect(nx);
	  }
     }
   else if (!strcmp(msg->msg, "node_add"))
     {
	Nexxus *nx;
	Node *nd;
	char nx_name[4096], nd_name[4096];
	
	sscanf(msg->dat, "%[^:]:%[^:]", nx_name, nd_name);
	nx = nexxus_find(nx_name);
	if (nx)
	  {
	     Node_Info *ni;
	     int i;
	     
	     nd = node_new(nx);
	     nd->name = g_strdup(nd_name);
	     ni = g_malloc(sizeof(Node_Info));
	     ni->noo = 1;
	     ni->dostat = 0;
	     for (i = 0; i < 128; i++) ni->stats.cpu_load[i] = 0.0;
	     for (i = 0; i < 60; i++) ni->stats.mem_used[i] = 0.0;
	     for (i = 0; i < 60; i++) ni->stats.swap_used[i] = 0.0;
	     nd->data = ni;
	  }
	if (nx)
	  {
	     GtkWidget *frame;
	     
	     frame = gtk_object_get_data(GTK_OBJECT(base), nx->name);
	     if (frame)
	       {
		  GtkWidget *vbox;

#define PANE_W 256
#define PANE_H 128
		  vbox = gtk_object_get_data(GTK_OBJECT(frame), "vbox");
		  if (vbox)
		    {
		       GtkWidget *vbox2, *box, *hbox, *button, *arrow, *label, *box2, *entry, *curve, *frame2, *scrolledwindow, *clist;
		       
		       vbox2 = gtk_notebook_new ();
		       
		       frame = gtk_frame_new(nd->name);
		       gtk_container_set_border_width (GTK_CONTAINER (frame), 2);
		       gtk_widget_show(frame);
		       gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
		       gtk_object_set_data(GTK_OBJECT(vbox), nd->name, frame);
		       
		       box = gtk_hbox_new (FALSE, 1);
		       gtk_widget_show(box);
		       gtk_container_add (GTK_CONTAINER (frame), box);
		       gtk_object_set_data(GTK_OBJECT (frame), "box", box);
		       
		       hbox = gtk_vbox_new (FALSE, 1);
		       gtk_widget_show(hbox);
		       gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 1);
		       
		       button = gtk_button_new();
		       gtk_widget_show(button);
		       gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 1);
		       
		       gtk_signal_connect (GTK_OBJECT (button), "clicked",
					   GTK_SIGNAL_FUNC (on_prev_clicked),
					   vbox2);
		       
		       arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_IN);
		       gtk_widget_show(arrow);
		       gtk_container_add (GTK_CONTAINER (button), arrow);
		       
		       button = gtk_button_new();
		       gtk_widget_show(button);
		       gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 1);
		       
		       gtk_signal_connect (GTK_OBJECT (button), "clicked",
					   GTK_SIGNAL_FUNC (on_next_clicked),
					   vbox2);
		       
		       arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_IN);
		       gtk_widget_show(arrow);
		       gtk_container_add (GTK_CONTAINER (button), arrow);
		       
		       gtk_notebook_set_scrollable(GTK_NOTEBOOK(vbox2), TRUE);
		       gtk_notebook_popup_enable(GTK_NOTEBOOK(vbox2));
		       gtk_widget_show (vbox2);	  
		       gtk_box_pack_start (GTK_BOX (box), vbox2, FALSE, FALSE, 1);
		       gtk_object_set_data(GTK_OBJECT (frame), "vbox2", vbox2);

		       
		       
		       
		       box2 = gtk_vbox_new (FALSE, 2);
		       gtk_widget_show(box2);

		       label = gtk_label_new(_("CPU Load"));
		       gtk_widget_show(label);
		       gtk_notebook_append_page(GTK_NOTEBOOK(vbox2), box2, label);
		       
		       entry = gtk_entry_new();
		       gtk_widget_show(entry);
		       gtk_widget_set_usize(entry, PANE_W, -1);
		       gtk_box_pack_start (GTK_BOX (box2), entry, FALSE, FALSE, 2);
		       
		       frame2 = gtk_frame_new(NULL);
		       gtk_widget_show(frame2);
		       gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
		       gtk_box_pack_start (GTK_BOX (box2), frame2, FALSE, FALSE, 2);

		       curve = gtk_curve_new();
		       gtk_widget_show(curve);
		       gtk_widget_set_usize(curve, PANE_W, PANE_H);
		       gtk_curve_set_range(GTK_CURVE(curve), 0, 127, 0, 100);
			 {
			    gfloat vals[3] = {0, 0, 0};
			    gtk_curve_set_vector (GTK_CURVE(curve), 3, vals);
			 }
		       gtk_container_add(GTK_CONTAINER(frame2), curve);
		       gtk_curve_set_curve_type(GTK_CURVE(curve), GTK_CURVE_TYPE_FREE);
		       gtk_object_set_data(GTK_OBJECT (frame), "cpu_entry", entry);
		       gtk_object_set_data(GTK_OBJECT (frame), "cpu_curve", curve);

		    
		       box2 = gtk_vbox_new (FALSE, 2);
		       gtk_widget_show(box2);

		       label = gtk_label_new(_("Memory Usage"));
		       gtk_widget_show(label);
		       gtk_notebook_append_page(GTK_NOTEBOOK(vbox2), box2, label);
		       
		       entry = gtk_entry_new();
		       gtk_widget_show(entry);
		       gtk_widget_set_usize(entry, PANE_W, -1);
		       gtk_box_pack_start (GTK_BOX (box2), entry, FALSE, FALSE, 2);
		       
		       hbox = gtk_hbox_new (FALSE, 0);
		       gtk_widget_show(hbox);
		       gtk_box_pack_start (GTK_BOX (box2), hbox, FALSE, FALSE, 2);
		       
		       frame2 = gtk_frame_new(NULL);
		       gtk_widget_show(frame2);
		       gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
		       gtk_box_pack_start (GTK_BOX (hbox), frame2, FALSE, FALSE, 2);

		       curve = gtk_curve_new();
		       gtk_widget_show(curve);
		       gtk_widget_set_usize(curve, (PANE_W - 8)/ 2, PANE_H);
		       gtk_curve_set_range(GTK_CURVE(curve), 0, 59, 0, 100);
			 {
			    gfloat vals[3] = {0, 0, 0};
			    gtk_curve_set_vector (GTK_CURVE(curve), 3, vals);
			 }
		       gtk_container_add(GTK_CONTAINER(frame2), curve);
		       gtk_curve_set_curve_type(GTK_CURVE(curve), GTK_CURVE_TYPE_FREE);
		       gtk_object_set_data(GTK_OBJECT (frame), "memory_curve", curve);

		       frame2 = gtk_frame_new(NULL);
		       gtk_widget_show(frame2);
		       gtk_frame_set_shadow_type(GTK_FRAME(frame2), GTK_SHADOW_IN);
		       gtk_box_pack_start (GTK_BOX (hbox), frame2, FALSE, FALSE, 2);

		       curve = gtk_curve_new();
		       gtk_widget_show(curve);
		       gtk_widget_set_usize(curve, (PANE_W - 8)/ 2, PANE_H);
		       gtk_curve_set_range(GTK_CURVE(curve), 0, 59, 0, 100);
			 {
			    gfloat vals[3] = {0, 0, 0};
			    gtk_curve_set_vector (GTK_CURVE(curve), 3, vals);
			 }
		       gtk_container_add(GTK_CONTAINER(frame2), curve);
		       gtk_curve_set_curve_type(GTK_CURVE(curve), GTK_CURVE_TYPE_FREE);
		       gtk_object_set_data(GTK_OBJECT (frame), "swap_curve", curve);
		       
		       gtk_object_set_data(GTK_OBJECT (frame), "memory_entry", entry);

		    
		    
		       box2 = gtk_vbox_new (FALSE, 2);
		       gtk_widget_show(box2);

		       label = gtk_label_new(_("Disk Usage"));
		       gtk_widget_show(label);
		       gtk_notebook_append_page(GTK_NOTEBOOK(vbox2), box2, label);
		       
		       scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
		       gtk_widget_show (scrolledwindow);
		       gtk_widget_set_usize(scrolledwindow, PANE_W, -1);
		       gtk_box_pack_start (GTK_BOX (box2), scrolledwindow, TRUE, TRUE, 2);
		       gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow), 4);
		       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

			 {
			    gchar *text[3] = {
			       "Device", "Mount", "%Usage"
			    };
			    clist = gtk_clist_new_with_titles (3, text);
			 }
		       gtk_widget_show (clist);
		       gtk_container_add (GTK_CONTAINER (scrolledwindow), clist);
		       gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_BROWSE);
		       
		       gtk_object_set_data(GTK_OBJECT (frame), "disks", clist);
		       
		       
		       box2 = gtk_vbox_new (FALSE, 2);
		       gtk_widget_show(box2);

		       label = gtk_label_new(_("Processes"));
		       gtk_widget_show(label);
		       gtk_notebook_append_page(GTK_NOTEBOOK(vbox2), box2, label);
		       
		       scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
		       gtk_widget_show (scrolledwindow);
		       gtk_widget_set_usize(scrolledwindow, PANE_W, -1);
		       gtk_box_pack_start (GTK_BOX (box2), scrolledwindow, TRUE, TRUE, 2);
		       gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow), 4);
		       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

			 {
			    gchar *text[6] = {
			       "Process", "PID", "User", "%CPU", "%MEM", "RSS"
			    };
			    clist = gtk_clist_new_with_titles (6, text);
			 }
		       gtk_widget_show (clist);
		       gtk_container_add (GTK_CONTAINER (scrolledwindow), clist);
		       gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_BROWSE);
		       
		       gtk_object_set_data(GTK_OBJECT (frame), "processes", clist);
		       
		    
		       box2 = gtk_vbox_new (FALSE, 2);
		       gtk_widget_show(box2);

		       label = gtk_label_new(_("Users logged on"));
		       gtk_widget_show(label);
		       gtk_notebook_append_page(GTK_NOTEBOOK(vbox2), box2, label);

		       scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
		       gtk_widget_show (scrolledwindow);
		       gtk_widget_set_usize(scrolledwindow, PANE_W, -1);
		       gtk_box_pack_start (GTK_BOX (box2), scrolledwindow, TRUE, TRUE, 2);
		       gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow), 4);
		       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

			 {
			    gchar *text[2] = {
			       "User", "TTY"
			    };
			    clist = gtk_clist_new_with_titles (2, text);
			 }
		       gtk_widget_show (clist);
		       gtk_container_add (GTK_CONTAINER (scrolledwindow), clist);
		       gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_BROWSE);
		       
		       gtk_object_set_data(GTK_OBJECT (frame), "users", clist);

		       
		       
		       
		       box2 = gtk_vbox_new (FALSE, 2);
		       gtk_widget_show(box2);

		       label = gtk_label_new(_("Misc"));
		       gtk_widget_show(label);
		       gtk_notebook_append_page(GTK_NOTEBOOK(vbox2), box2, label);
		       
		       label = gtk_label_new(_("Uptime: ???"));
		       gtk_widget_show(label);
		       gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
		       gtk_object_set_data(GTK_OBJECT (frame), "uptime", label);
		    }
	       }
	  }
     }
   else if (!strcmp(msg->msg, "node_del"))
     {
	Node *nd;
	GtkWidget *w, *vbox;
	
	nd = node_find(msg->dat);
	if (nd)
	  {
	     Node_Info *ni;
	     
	     ni = nd->data;
	     g_free(ni);
	     w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
	     vbox = gtk_object_get_data(GTK_OBJECT(w), "vbox");
	     w = gtk_object_get_data(GTK_OBJECT(vbox), nd->name);
	     gtk_widget_destroy(w);
	     node_free(nd);
	  }
     }
   else if (!strcmp(msg->msg, "node_rename"))
     {
	GtkWidget *w, *vbox;
	Node *nd;
        char nx_name[4096], nd_name[4096], nd_name2[4096], buf[4096];
	
	sscanf(msg->dat, "%[^:]:%[^:]:%[^:]", nx_name, nd_name, nd_name2);
	g_snprintf(buf, sizeof(buf), "%s:%s", nx_name, nd_name);
	nd = node_find(buf);
	if (nd)
	  {
	     if (nd->name) g_free(nd->name);
	     nd->name = g_strdup(nd_name2);
	     w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
	     vbox = gtk_object_get_data(GTK_OBJECT(w), "vbox");
	     w = gtk_object_get_data(GTK_OBJECT(vbox), nd_name);
	     gtk_object_remove_data(GTK_OBJECT(base), nd_name);
	     gtk_object_set_data(GTK_OBJECT(vbox), nd->name, w);
	     gtk_frame_set_label(GTK_FRAME(w), nd->name);
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_select"))
     {
	Nexxus *nx;
	
	nx = nexxus_find(msg->dat);
	if (nx) 
	  {
	     nx->selected = 1;
	  }
     }
   else if (!strcmp(msg->msg, "node_select"))
     {
	Node *nd;
	
	nd = node_find(msg->dat);
	if (nd) 
	  {
	     nd->selected = 1;
	  }
     }
   else if (!strcmp(msg->msg, "nexxus_unselect"))
     {
	Nexxus *nx;
	
	nx = nexxus_find(msg->dat);
	if (nx) 
	  {
	     nx->selected = 0;
	  }
     }
   else if (!strcmp(msg->msg, "node_unselect"))
     {
	Node *nd;
	
	nd = node_find(msg->dat);
	if (nd) 
	  {
	     nd->selected = 0;
	  }
     }
}

gint
poll_timeout(gpointer data)
{
   GList *l;
   int i;
   int didnoo = 0;
   
   flim_client_block_messages();
   for (l = nexxus_list; l; l = l->next)
     {
	Nexxus *nx;
	GList *ll;
	
	nx = l->data;
	for (ll = nx->nodes; ll; ll = ll->next)
	  {
	     Node *nd;
	     Node_Info *in;
	     
	     nd = ll->data;
	     in = nd->data;
	     /* new node - figure out if it does sensors */
	     if ((in->noo) && (nx->connection))
	       {
		  char buf[4096];
		  
		  g_snprintf(buf, sizeof(buf), "SYSSTAT:MEM:%s", nd->name);
		  FLIM_COMMS_BEGIN(nx->connection, buf);
		  if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
		    {
		       while (FLIM_COMMS_REPLY)
			 {
			    if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			       break;
			    if ((FLIM_COMMS_REPLY_COUNT >= 3) &&
				(!strcmp("MEM", FLIM_COMMS_REPLY[2])))
			       in->dostat = 1;
			    FILM_COMMS_GET_REPLY;
			 }
		    }
		  FILM_COMMS_REPLY_END;
		  in->noo = 0;
		  if (!in->dostat)
		    {
		       GtkWidget *w, *box, *label;
		       
		       w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
		       w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
		       w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
		       box = gtk_object_get_data(GTK_OBJECT(w), "box");
		       gtk_widget_destroy(box);
		       label = gtk_label_new(_("No Sysstat support on Node"));
		       gtk_widget_show(label);
		       gtk_container_add(GTK_CONTAINER(w), label);
		    }
		  didnoo = 1;
	       }
	  }
     }   
   if (didnoo)
     {
	gtk_widget_hide(base);
	gtk_widget_show(base);
     }
   for (l = nexxus_list; l; l = l->next)
     {
	Nexxus *nx;
	GList *ll;
	GList *jobs;
	
	jobs = NULL;
	nx = l->data;
	for (ll = nx->nodes; ll; ll = ll->next)
	  {
	     Node *nd;
	     Node_Info *in;
	     
	     nd = ll->data;
	     in = nd->data;
	     if ((in->dostat) && (nx->connection))
	       {
		  char buf[4096];
		  GtkWidget *w, *ww, *www;
		  
		  w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
		  w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
		  w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
		  w = gtk_object_get_data(GTK_OBJECT(w), "users");
		  gtk_clist_freeze(GTK_CLIST(w));
		  gtk_clist_clear(GTK_CLIST(w));
		  g_snprintf(buf, sizeof(buf), "SYSSTAT:WHO:%s", nd->name);
		  FLIM_COMMS_BEGIN(nx->connection, buf);
		  if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
		    {
		       while (FLIM_COMMS_REPLY)
			 {
			    if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			       break;
			    if ((FLIM_COMMS_REPLY_COUNT >= 5) &&
				(!strcmp("WHO", FLIM_COMMS_REPLY[2])))
			      {
				 gchar *text[2];
				 
				 text[0] = FLIM_COMMS_REPLY[3];
				 text[1] = FLIM_COMMS_REPLY[4];
				 gtk_clist_append(GTK_CLIST(w), text);
			      }
			    FILM_COMMS_GET_REPLY;
			 }
		    }
		  FILM_COMMS_REPLY_END;
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 0, TRUE);
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 1, TRUE);
		  gtk_clist_thaw(GTK_CLIST(w));

		  w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
		  w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
		  w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
		  w = gtk_object_get_data(GTK_OBJECT(w), "disks");
		  gtk_clist_freeze(GTK_CLIST(w));
		  gtk_clist_clear(GTK_CLIST(w));
		  g_snprintf(buf, sizeof(buf), "SYSSTAT:FS:%s", nd->name);
		  FLIM_COMMS_BEGIN(nx->connection, buf);
		  if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
		    {
		       while (FLIM_COMMS_REPLY)
			 {
			    if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			       break;
			    if ((FLIM_COMMS_REPLY_COUNT >= 9) &&
				(!strcmp("FS", FLIM_COMMS_REPLY[2])))
			      {
				 gchar *text[3];
				 
				 text[0] = FLIM_COMMS_REPLY[3];
				 text[1] = FLIM_COMMS_REPLY[8];
				 text[2] = FLIM_COMMS_REPLY[7];
				 gtk_clist_append(GTK_CLIST(w), text);
			      }
			    FILM_COMMS_GET_REPLY;
			 }
		    }
		  FILM_COMMS_REPLY_END;
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 0, TRUE);
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 1, TRUE);
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 2, TRUE);
		  gtk_clist_thaw(GTK_CLIST(w));

		  w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
		  w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
		  w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
		  w = gtk_object_get_data(GTK_OBJECT(w), "uptime");
		  g_snprintf(buf, sizeof(buf), "SYSSTAT:UPTIME:%s", nd->name);
		  FLIM_COMMS_BEGIN(nx->connection, buf);
		  if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
		    {
		       while (FLIM_COMMS_REPLY)
			 {
			    if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			       break;
			    if ((FLIM_COMMS_REPLY_COUNT >= 4) &&
				(!strcmp("UPTIME", FLIM_COMMS_REPLY[2])))
			      {
				 gint secs, mins, hrs, days, weeks;
				 gchar buf[256];
				 
				 secs = atoi(FLIM_COMMS_REPLY[3]);
				 mins = secs / 60;
				 hrs = mins / 60;
				 days = hrs / 24;
				 weeks = days / 7;
				 
				 secs -= (mins * 60);
				 mins -= (hrs * 60);
				 hrs -= (days * 24);
				 days -= (weeks * 7);
				 
				 g_snprintf(buf, sizeof(buf), "Uptime: %i weeks, %i days, %i:%02i:%02i",
					    weeks, days, hrs, mins, secs);
				 gtk_label_set_text(GTK_LABEL(w), buf);
			      }
			    FILM_COMMS_GET_REPLY;
			 }
		    }
		  FILM_COMMS_REPLY_END;

		  w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
		  w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
		  w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
		  ww = gtk_object_get_data(GTK_OBJECT(w), "memory_curve");
		  www = gtk_object_get_data(GTK_OBJECT(w), "swap_curve");
		  w = gtk_object_get_data(GTK_OBJECT(w), "memory_entry");
		  g_snprintf(buf, sizeof(buf), "SYSSTAT:MEM:%s", nd->name);
		  FLIM_COMMS_BEGIN(nx->connection, buf);
		  if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
		    {
		       while (FLIM_COMMS_REPLY)
			 {
			    if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			       break;
			    if ((FLIM_COMMS_REPLY_COUNT >= 10) &&
				(!strcmp("MEM", FLIM_COMMS_REPLY[2])))
			      {
				 gchar buf[256];
				 int mem_total, swap_total, mem_free, mem_shared, mem_buf, mem_cache, swap_free;
				 
				 mem_total = atoi(FLIM_COMMS_REPLY[3]);
				 mem_free = atoi(FLIM_COMMS_REPLY[4]);
				 mem_shared = atoi(FLIM_COMMS_REPLY[5]);
				 mem_buf = atoi(FLIM_COMMS_REPLY[6]);
				 mem_cache = atoi(FLIM_COMMS_REPLY[7]);
				 swap_total = atoi(FLIM_COMMS_REPLY[8]);
				 swap_free = atoi(FLIM_COMMS_REPLY[9]);
				 
				 g_snprintf(buf, sizeof(buf),
					    "MEM (Kb): %i/%i (%i Sh, %i Buf, %i Cache) SWAP: %i/%i",
					    (mem_total - mem_free - mem_buf - mem_cache),
					    (mem_total),
					    mem_shared,
					    mem_buf,
					    mem_cache,
					    (swap_total - swap_free),
					    swap_total);
				 gtk_entry_set_text(GTK_ENTRY(w), buf);
				 for (i = 0; i < 59; i++)
				    in->stats.mem_used[i] = in->stats.mem_used[i + 1];
				 in->stats.mem_used[59] = 100 * ((double)(mem_total - mem_free - mem_buf - mem_cache) / (double)mem_total);
				 gtk_curve_set_vector (GTK_CURVE(ww), 60, in->stats.mem_used);
				 for (i = 0; i < 59; i++)
				    in->stats.swap_used[i] = in->stats.swap_used[i + 1];
				 in->stats.swap_used[59] = 100 * ((double)(swap_total - swap_free) / (double)swap_total);
				 gtk_curve_set_vector (GTK_CURVE(www), 60, in->stats.swap_used);
			      }
			    FILM_COMMS_GET_REPLY;
			 }
		    }
		  FILM_COMMS_REPLY_END;

		  w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
		  w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
		  w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
		  w = gtk_object_get_data(GTK_OBJECT(w), "processes");
		  gtk_clist_freeze(GTK_CLIST(w));
		  gtk_clist_clear(GTK_CLIST(w));
		  g_snprintf(buf, sizeof(buf), "SYSSTAT:PS:%s", nd->name);
		  FLIM_COMMS_BEGIN(nx->connection, buf);
		  if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
		    {
		       while (FLIM_COMMS_REPLY)
			 {
			    if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			       break;
			    if ((FLIM_COMMS_REPLY_COUNT >= 11) &&
				(!strcmp("PS", FLIM_COMMS_REPLY[2])))
			      {
				 gchar *text[6];
				 
				 text[0] = FLIM_COMMS_REPLY[10];
				 text[1] = FLIM_COMMS_REPLY[3];
				 text[2] = FLIM_COMMS_REPLY[4];
				 text[3] = FLIM_COMMS_REPLY[5];
				 text[4] = FLIM_COMMS_REPLY[6];
				 text[5] = FLIM_COMMS_REPLY[7];
				 gtk_clist_append(GTK_CLIST(w), text);
			      }
			    FILM_COMMS_GET_REPLY;
			 }
		    }
		  FILM_COMMS_REPLY_END;
		  flim_comms_do_gui(1);
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 0, TRUE);
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 1, TRUE);
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 2, TRUE);
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 3, TRUE);
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 4, TRUE);
		  gtk_clist_set_column_auto_resize(GTK_CLIST(w), 5, TRUE);
		  gtk_clist_thaw(GTK_CLIST(w));

		  w = gtk_object_get_data(GTK_OBJECT(base), nd->nexxus->name);
		  w = gtk_object_get_data(GTK_OBJECT(w), "vbox");
		  w = gtk_object_get_data(GTK_OBJECT(w), nd->name);
		  ww = gtk_object_get_data(GTK_OBJECT(w), "cpu_curve");
		  w = gtk_object_get_data(GTK_OBJECT(w), "cpu_entry");
		  g_snprintf(buf, sizeof(buf), "SYSSTAT:CPU_LOAD:%s", nd->name);
		  FLIM_COMMS_BEGIN(nx->connection, buf);
		  if ((FLIM_COMMS_REPLY) && (FLIM_COMMS_REPLY_COUNT >= 0))
		    {
		       while (FLIM_COMMS_REPLY)
			 {
			    if (!strcmp("JOB_COMPLETED", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("MODULE_NOT_FOUND", FLIM_COMMS_REPLY[2]))
			       break;
			    if (!strcmp("JOB_ERROR", FLIM_COMMS_REPLY[2]))
			       break;
			    if ((FLIM_COMMS_REPLY_COUNT >= 6) &&
				(!strcmp("CPU_LOAD", FLIM_COMMS_REPLY[2])))
			      {
				 gchar buf[256];
				 gfloat v;
				 
				 g_snprintf(buf, sizeof(buf), "(1min) %s (5min) %s (15min) %s",
					    FLIM_COMMS_REPLY[3], FLIM_COMMS_REPLY[4], FLIM_COMMS_REPLY[5]);
				 gtk_entry_set_text(GTK_ENTRY(w), buf);
				 v = atof(FLIM_COMMS_REPLY[3]);
				 for (i = 0; i < 127; i++)
				    in->stats.cpu_load[i] = in->stats.cpu_load[i + 1];
				 in->stats.cpu_load[127] = v * 25;
				 gtk_curve_set_vector (GTK_CURVE(ww), 128, in->stats.cpu_load);
			      }
			    FILM_COMMS_GET_REPLY;
			 }
		    }
		  FILM_COMMS_REPLY_END;
	       }
	  }
     }
   data = NULL;
   gtk_timeout_add(10000, poll_timeout, NULL);
   flim_client_unblock_messages();
   return 0;
}

GtkWidget *
create_base(void)
{
   GtkWidget *base;
   GtkWidget *vbox1;
   GtkWidget *entry;
   
   base = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_object_set_data (GTK_OBJECT (base), "base", base);
   gtk_container_set_border_width (GTK_CONTAINER (base), 2);
   gtk_window_set_title (GTK_WINDOW (base), "Flim");
   gtk_window_set_wmclass (GTK_WINDOW (base), "Sensors", "FLIM");
   gtk_window_set_policy (GTK_WINDOW (base), 0, 1, 1);
   
   vbox1 = gtk_vbox_new (FALSE, 0);
   gtk_widget_ref (vbox1);
   gtk_object_set_data(GTK_OBJECT (base), "vbox1", vbox1);
   gtk_widget_show (vbox1);
   gtk_container_add (GTK_CONTAINER (base), vbox1);

   entry = gtk_entry_new();
   gtk_widget_show(entry);
   gtk_widget_set_usize(entry, -1, -1);
   gtk_box_pack_start (GTK_BOX (vbox1), entry, FALSE, FALSE, 2);
   gtk_entry_set_editable(GTK_ENTRY(entry), 0);
   gtk_entry_set_text(GTK_ENTRY(entry), _("System Statistics"));
   
   gtk_signal_connect (GTK_OBJECT (base), "destroy",
		       GTK_SIGNAL_FUNC (exit),
		       NULL);
   
   return base;
}

int
main(int argc, char **argv)
{
   Msg msg;
   
   flim_client_init(argc, argv);
#ifdef ENABLE_NLS
   bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
   textdomain (PACKAGE);
#endif
   
   gtk_set_locale ();
   gtk_init (&argc, &argv);
   
   
   base = create_base();
   gtk_widget_realize(base);

   g_snprintf(msg.msg, sizeof(msg.msg), "window_id");
   g_snprintf(msg.dat, sizeof(msg.dat), "%i", (int)GDK_WINDOW_XWINDOW(base->window));
   flim_client_send(&msg);

   for (;;) { XEvent xev; if (XCheckTypedWindowEvent(GDK_WINDOW_XDISPLAY(base->window), GDK_WINDOW_XWINDOW(base->window), ReparentNotify, &xev)) break; usleep(100000); }
   gtk_widget_show(base);
   
   g_snprintf(msg.msg, sizeof(msg.msg), "ready");
   flim_client_send(&msg);

   flim_comms_do_gui(1);   
   gtk_timeout_add(10000, poll_timeout, NULL);
   
   gtk_main();
   return 0;
}
