#include "main.h"

GList *groups_list = NULL;

void
flim_group_load(void)
{
   gchar *home, buf[4096];
   FILE *f;
   int next_name = 0, next_nexxus = 0, next_node = 0;
   Group *g = NULL;
   Nexxus *nx;
   Node *nd;
   GList *l, *ll;
   
   home = getenv("HOME");
   if (!home) return;
   g_snprintf(buf, sizeof(buf), "%s/.flim/groups", home);
   f = fopen(buf, "r");
   if (!f)
      return;
   while (fgets(buf, sizeof(buf) - 1, f))
     {
	char s1[4096], s2[4096];
	
	if (strlen(buf) > 0)
	  {
	     buf[strlen(buf) - 1] = 0;
	  }
	sscanf(buf, "%s %s", s1, s2);
	if (next_name)
	  {
	     if (!g)
		g = flim_group_new();
	     g->name = g_strdup(buf);
	     next_name = 0;
	  }
	else if (next_node)
	  {
	     if (!g)
		g = flim_group_new();
	     next_node = 0;
	     for (l = nexxus_list; l; l = l->next)
	       {
		  nx = l->data;
		  
		  for (ll = nx->nodes; ll; ll = ll->next)
		    {
		       nd = ll->data;
		       
		       if (!strcmp(nd->name, buf))
			 {
			    flim_group_add_member(g, nd);
			    goto done;
			 }
		    }
	       }
	     done: ;
	  }
	else if (next_nexxus)
	  {
	     if (!g)
		g = flim_group_new();
	     for (l = nexxus_list; l; l = l->next)
	       {
		  nx = l->data;
		  if (!strcmp(nx->name, buf))
		     flim_group_add_member(g, nx);
	       }
	     next_nexxus = 0;
	  }
	else if (!strcmp("::::", s1))
	  {
	     if (!strcmp("group", s2)) next_name = 1;
	     if (!strcmp("nexxus", s2)) next_nexxus = 1;
	     if (!strcmp("node", s2)) next_node = 1;
	     if (!strcmp("end", s2)) g = NULL;
	  }
     }
   if (!f)
      return;
   fclose(f);
}

void
flim_group_save(void)
{
   gchar *home, buf[4096];
   GList *l;
   FILE *f;
   
   home = getenv("HOME");
   if (!home) return;
   g_snprintf(buf, sizeof(buf), "%s/.flim", home);
   mkdir(buf, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
   g_snprintf(buf, sizeof(buf), "%s/.flim/groups", home);
   f = fopen(buf, "w");
   if (!f)
      return;
   for (l = groups_list; l; l = l->next)
     {
	Group *g;
	GList *ll;
	
	g = l->data;
	fprintf(f, ":::: group\n");
	fprintf(f, "%s\n", g->name);
	for (ll = g->members; ll; ll = ll->next)
	  {
	     Nexxus *nx;
	     Node *nd;
	     
	     nx = ll->data;
	     nd = ll->data;
	     if (nx->type == 0) /* nexxus */
	       {
		  fprintf(f, ":::: nexxus\n");
		  fprintf(f, "%s\n", nx->name);
	       }
	     else /* node */
	       {
		  fprintf(f, ":::: node\n");
		  fprintf(f, "%s\n", nd->name);
	       }
	  }
	fprintf(f, ":::: end\n");
     }
   fclose(f);
}

Group *
flim_group_new(void)
{
   Group *g;
   
   g = g_malloc(sizeof(Group));
   g->name = NULL;
   g->members = NULL;
   g->selected = 0;
   groups_list = g_list_append(groups_list, g);
   return g;
}

Group *
flim_group_find(char *name)
{
   GList *l;
   
  for (l = groups_list; l; l = l->next)
     {
	Group *g;
     
	g = l->data;
	if ((g->name) && (name) && (!strcmp(g->name, name)))
	   return g;
     }
   return NULL;
}

void
flim_group_add_member(Group *g, void *member)
{
   g->members = g_list_append(g->members, member);
}

void
flim_group_del_member(Group *g, void *member)
{
   g->members = g_list_remove(g->members, member);
}

void
flim_group_free(Group *g)
{
   groups_list = g_list_remove(groups_list, g);
   if (g->name)
      g_free(g->name);
   if (g->members)
      g_list_free(g->members);
   g_free(g);
}

void
flim_group_purge_member(void *member)
{
   GList *l;
   
   for (l = groups_list; l; l = l->next)
     {
	Group *g;
	
	g = l->data;
	flim_group_del_member(g, member);
     }
}

void
flim_group_select(GtkWidget *tree)
{
   GList *l;
   
   for (l = nexxus_list; l; l = l->next)
     {
	GList *ll;
	Nexxus *nx;
	nx = l->data;
	
	for (ll = nx->nodes; ll; ll = ll->next)
	  {
	     Node *nd;
	     GtkCTreeNode *ctree_node;
	     
	     nd = ll->data;
	     ctree_node = gtk_ctree_find_by_row_data(GTK_CTREE(tree), NULL, ll->data);
	     if (ctree_node)
	       {
		  if (nd->selected)
		     gtk_ctree_unselect(GTK_CTREE(tree), ctree_node);
	       }
	     
	  }
     }
   for (l = groups_list; l; l = l->next)
     {
	GList *ll;
	Group *g;
	
	g = l->data;
	for (ll = g->members; ll; ll = ll->next)
	  {
	     GtkCTreeNode *ctree_node;
	     
	     ctree_node = gtk_ctree_find_by_row_data(GTK_CTREE(tree), NULL, ll->data);
	     if (ctree_node)
	       {
		  if (g->selected)
		     gtk_ctree_select(GTK_CTREE(tree), ctree_node);
	       }
	  }
     }
}

void
flim_group_build_list(GtkWidget *list)
{
   GList *l;
   
   gtk_clist_freeze(GTK_CLIST(list));
   gtk_clist_clear(GTK_CLIST(list));
   for (l = groups_list; l; l = l->next)
     {
	Group *g;
	gchar *text[1];
	gint num;
	
	g = l->data;
	
	g->selected = 0;
	text[0] = g->name;
	num = gtk_clist_append(GTK_CLIST(list), text);
	gtk_clist_set_row_data(GTK_CLIST(list), num, g);
     }
   gtk_clist_thaw(GTK_CLIST(list));
}
