#include "main.h"

void flim_client_message(Msg *msg);

GList *plugin_list = NULL;
GList *installed_toolbar_plugins = NULL;
GList *installed_plugins = NULL;
static int msg_id = -1;
static int plugin_id = 1;

typedef struct _msgbuf Flim_MsgBuf;

struct _msgbuf
{
   long mtype;
   char mtext[5120];
};

static gint
flim_plugin_msg_timeout(gpointer data)
{
   data = NULL;
   flim_plugin_handle_messages();
   return 1;
}

static void
flim_plugin_cleanup(void)
{
   msgctl(msg_id, IPC_RMID, NULL);
}

void
flim_plugin_init(void)
{
   atexit(flim_plugin_cleanup);
   msg_id = msgget(ftok(getenv("HOME"), getpid()), IPC_CREAT | 0666);
   gtk_timeout_add(50, flim_plugin_msg_timeout, NULL);
}

void
flim_plugin_load_config(void)
{
   gchar *home, buf[4096];
   FILE *f;
   int   read_plugins, read_toolbar_plugin, read_plugin, read_active;
   
   home = getenv("HOME");
   if (!home) return;
   g_snprintf(buf, sizeof(buf), "%s/.flim/plugins", home);
   f = fopen(buf, "r");
   if (!f)
     {
	/* no previous plugins */
	Installed_Plugin *i_plugin;
	
	i_plugin = g_malloc(sizeof(Installed_Plugin));
	i_plugin->path = g_strdup(LIBDIR"/flim/toolbar.p");
	i_plugin->active = 1;
	installed_toolbar_plugins = g_list_append(installed_toolbar_plugins, i_plugin);

	i_plugin = g_malloc(sizeof(Installed_Plugin));
	i_plugin->path = g_strdup(LIBDIR"/flim/admin.p");
	i_plugin->active = 1;
	installed_toolbar_plugins = g_list_append(installed_plugins, i_plugin);

	i_plugin = g_malloc(sizeof(Installed_Plugin));
	i_plugin->path = g_strdup(LIBDIR"/flim/sercon.p");
	i_plugin->active = 1;
	installed_toolbar_plugins = g_list_append(installed_plugins, i_plugin);

	/*..*/
	
	i_plugin = g_malloc(sizeof(Installed_Plugin));
	i_plugin->path = g_strdup(LIBDIR"/flim/chassis.p");
	i_plugin->active = 1;
	installed_plugins = g_list_append(installed_plugins, i_plugin);

	i_plugin = g_malloc(sizeof(Installed_Plugin));
	i_plugin->path = g_strdup(LIBDIR"/flim/sensors.p");
	i_plugin->active = 1;
	installed_plugins = g_list_append(installed_plugins, i_plugin);

	i_plugin = g_malloc(sizeof(Installed_Plugin));
	i_plugin->path = g_strdup(LIBDIR"/flim/systat.p");
	i_plugin->active = 1;
	installed_plugins = g_list_append(installed_plugins, i_plugin);
	
	return;
     }
   read_plugins = 0;
   read_toolbar_plugin = 0;
   read_plugin = 0;
   read_active = 0;
   while (fgets(buf, sizeof(buf) - 1, f))
     {  
	char s1[4096], s2[4096];
	
        if (strlen(buf) > 0) buf[strlen(buf) - 1] = 0;
	s1[0] = 0; s2[0] = 0;
	sscanf(buf, "%s %s", s1, s2);
	if ((!strcmp(s1, "::::")) && (!strcmp(s2, "plugins")))
	   read_plugins = 1;
	else if ((read_plugins) && (!strcmp(s1, "::::")) && (!strcmp(s2, "plugin")))
	   read_plugin = 1;   
	else if ((read_plugins) && (!strcmp(s1, "::::")) && (!strcmp(s2, "toolbar_plugin")))
	   read_toolbar_plugin = 1;
	else if ((read_plugins) && (!strcmp(s1, "::::")) && (!strcmp(s2, "active_plugin")))
	  {
	     read_plugin = 1;   
	     read_active = 1;
	  }
	else if ((read_plugins) && (!strcmp(s1, "::::")) && (!strcmp(s2, "active_toolbar_plugin")))
	  {
	     read_toolbar_plugin = 1;
	     read_active = 1;
	  }
	else if (read_plugin)
	  {
	     Installed_Plugin *i_plugin;
	     
	     i_plugin = g_malloc(sizeof(Installed_Plugin));
	     i_plugin->path = g_strdup(buf);
	     i_plugin->active = read_active;
	     installed_plugins = g_list_append(installed_plugins, i_plugin);
	     read_plugin = 0;
	     read_active = 0;
	  }
	else if (read_toolbar_plugin)
	  {
	     Installed_Plugin *i_plugin;
	     
	     i_plugin = g_malloc(sizeof(Installed_Plugin));
	     i_plugin->path = g_strdup(buf);
	     i_plugin->active = read_active;
	     installed_toolbar_plugins = g_list_append(installed_toolbar_plugins, i_plugin);
	     read_toolbar_plugin = 0;
	     read_active = 0;
	  }
     }
   fclose(f);
}

void
flim_plugin_save_config(void)
{
   gchar *home, buf[4096];
   FILE *f;
   GList *l;
   
   home = getenv("HOME");
   if (!home) return;
   g_snprintf(buf, sizeof(buf), "%s/.flim/plugins", home);
   f = fopen(buf, "w");
   if (!f)
      return;
   fprintf(f, ":::: plugins\n");
   for (l = installed_plugins; l; l = l->next)
     {
	Installed_Plugin *i_plugin;
	
	i_plugin = l->data;
	if (i_plugin->active)
	  {
	     fprintf(f, ":::: active_plugin\n");
	     fprintf(f, "%s\n", i_plugin->path);
	  }
	else
	  {
	     fprintf(f, ":::: plugin\n");
	     fprintf(f, "%s\n", i_plugin->path);
	  }
     }
   for (l = installed_toolbar_plugins; l; l = l->next)
     {
	Installed_Plugin *i_plugin;
	
	i_plugin = l->data;
	if (i_plugin->active)
	  {
	     fprintf(f, ":::: active_toolbar_plugin\n");
	     fprintf(f, "%s\n", i_plugin->path);
	  }
	else
	  { 
	     fprintf(f, ":::: toolbar_plugin\n");
	     fprintf(f, "%s\n", i_plugin->path);
	  }
     }
   fclose(f);
}

void
flim_start_plugins(void)
{
   GList *l;
   
   for (l = installed_toolbar_plugins; l; l = l->next)
     {
	Installed_Plugin *i_plugin;
	
	i_plugin = l->data;
	if (i_plugin->active)
	   flim_plugin_start(i_plugin->path);
     }
   for (l = installed_plugins; l; l = l->next)
     {
	Installed_Plugin *i_plugin;
	
	i_plugin = l->data;
	if (i_plugin->active)
	   flim_plugin_start(i_plugin->path);
     }
}

Installed_Plugin *
flim_find_plugin(char *path)
{
   GList *l;
   
   for (l = installed_plugins; l; l = l->next)
     {
	Installed_Plugin *i_plugin;
	
	i_plugin = l->data;
	if (!strcmp(i_plugin->path, path)) return i_plugin;
     }
   for (l = installed_toolbar_plugins; l; l = l->next)
     {
	Installed_Plugin *i_plugin;
	
	i_plugin = l->data;
	if (!strcmp(i_plugin->path, path)) return i_plugin;
     }
   return NULL;
}

void
flim_plugin_add(char *path)
{
   Installed_Plugin *i_plugin;
   
   i_plugin = g_malloc(sizeof(Installed_Plugin));
   i_plugin->path = g_strdup(path);
   i_plugin->active = 0;
   installed_plugins = g_list_append(installed_plugins, i_plugin);
}

void
flim_plugin_del(char *path)
{
   Installed_Plugin *i_plugin;
   
   i_plugin = flim_find_plugin(path);
   if (i_plugin)
     {
	installed_plugins = g_list_remove(installed_plugins, i_plugin);
	if (i_plugin->path)
	   g_free(i_plugin->path);
	g_free(i_plugin);
     }
}

void
flim_toolbar_plugin_add(char *path)
{
   Installed_Plugin *i_plugin;
   
   i_plugin = g_malloc(sizeof(Installed_Plugin));
   i_plugin->path = g_strdup(path);
   i_plugin->active = 0;
   installed_toolbar_plugins = g_list_append(installed_toolbar_plugins, i_plugin);
}

void
flim_toolbar_plugin_del(char *path)
{
   Installed_Plugin *i_plugin;
   
   i_plugin = flim_find_plugin(path);
   if (i_plugin)
     {
	installed_toolbar_plugins = g_list_remove(installed_toolbar_plugins, i_plugin);
	if (i_plugin->path)
	   g_free(i_plugin->path);
	g_free(i_plugin);
     }
}


void
flim_plugin_update_lists(GtkWidget *toolbar_list, GtkWidget *plugin_list)
{
   GList *l;
   
   gtk_clist_clear(GTK_CLIST(toolbar_list));
   for (l = installed_toolbar_plugins; l; l = l->next)
     {
	Installed_Plugin *i_plugin;
	gchar *txt[1];
	gint row;
	char *last;
	
	i_plugin = l->data;
	last = strrchr(i_plugin->path, '/');
	if (last) last++;
	else last = i_plugin->path;
	txt[0] = last;
	row = gtk_clist_append(GTK_CLIST(toolbar_list), txt);
	if (i_plugin->active)
	   gtk_clist_select_row(GTK_CLIST(toolbar_list), row, 0);
     }
   gtk_clist_clear(GTK_CLIST(plugin_list));
   for (l = installed_plugins; l; l = l->next)
     {
	Installed_Plugin *i_plugin;
	gchar *txt[1];
	gint row;
	char *last;
	
	i_plugin = l->data;
	last = strrchr(i_plugin->path, '/');
	if (last) last++;
	else last = i_plugin->path;
	txt[0] = last;
	row = gtk_clist_append(GTK_CLIST(plugin_list), txt);
	if (i_plugin->active)
	   gtk_clist_select_row(GTK_CLIST(plugin_list), row, 0);
     }
}

Plugin *
flim_plugin_start(char *name)
{
   Plugin *p;
   char exe[4096];

   p = g_malloc(sizeof(Plugin));
   plugin_list = g_list_append(plugin_list, p);
   p->name = g_strdup(name);
   p->id = plugin_id++;
   
   p->ready = 0;
   p->is_toolbar = 0;
   p->pid = 0;
   p->msg_id = -1;
   p->windows = NULL;
   p->pid = fork();
   
   p->dead = 0;
      
   if (p->pid)
      return p;

   g_snprintf(exe, sizeof(exe), "%s %i %i", name, msg_id, p->id);
   execl("/bin/sh", "/bin/sh", "-c", exe, NULL);
   exit(0);
   return 0;
}

Plugin *
flim_plugin_find(char *name)
{
   GList *l;
   
   for (l = plugin_list; l; l = l->next)
     {
	Plugin *p;
	
	p = l->data;
	if (!strcmp(p->name, name)) return p;
     }
   return NULL;
}

Plugin *
flim_plugin_find_pid(pid_t pid)
{
   GList *l;
   
   for (l = plugin_list; l; l = l->next)
     {
	Plugin *p;
	
	p = l->data;
	if (p->pid == pid) return p;
     }
   return NULL;
}

void
flim_plugin_stop(Plugin *plugin)
{
   if (!plugin) return;
   kill(plugin->pid, SIGTERM);
   plugin_list = g_list_remove(plugin_list, plugin);
   if (plugin->name) g_free(plugin->name);
   g_free(plugin);
}

void
flim_plugin_mark_dead(Plugin *plugin)
{
   plugin->dead = 1;
}

void
flim_plugin_send(Plugin *plugin, Msg *msg)
{
   Flim_MsgBuf msgp;

   if (plugin->msg_id < 0) return;
   if (plugin->dead) return;
   msgp.mtype = 1;
   memcpy(&(msgp.mtext[0]), msg, sizeof(Msg));
   msgsnd(plugin->msg_id, &msgp, sizeof(Msg), 0);
}

void
flim_plugin_broadcast(Msg *msg)
{
   GList *l;
   
   for (l = plugin_list; l; l = l->next)
     {
	Plugin *p;
	
	p = l->data;
	if ((!p->dead) && (p->ready))
	   flim_plugin_send(p, msg);
     }
}

void
flim_plugin_message(Plugin *plugin, Msg *msg)
{
   if (!strcmp(msg->msg, "return_ipc"))
     {
	plugin->msg_id = atoi(msg->dat);
     }
   else if (!strcmp(msg->msg, "nexxus_state"))
     {
     }
   else if (!strcmp(msg->msg, "ready"))
     {
	plugin->ready = 1;
	/* send plugin a list of current nexxus's and nodes */
	  {
	     GList *l;
	     
	     for (l = nexxus_list; l; l = l->next)
	       {
		  Nexxus *nx;
		  GList *ll;
		  Msg msg;
		  char *name, *address, *login, *password;
		  
		  nx = l->data;
		  g_snprintf(msg.msg, sizeof(msg.msg), "nexxus_add");
		  name = nx->name;
		  address = nx->address;
		  login = nx->login;
		  password = nx->password;
		  if (!name) name = "";
		  if (!address) address = "";
		  if (!login) login = "";
		  if (!password) password = "";   
		  g_snprintf(msg.dat, sizeof(msg.dat), "%s:%s:%s:%s", name, address, login, password);
		  flim_plugin_send(plugin, &msg);
		  for (ll = nx->nodes; ll; ll = ll->next)
		    {
		       Node *nd;
		       
		       nd = ll->data;
		       g_snprintf(msg.msg, sizeof(msg.msg), "node_add");
		       g_snprintf(msg.dat, sizeof(msg.dat), "%s:%s", nd->nexxus->name, nd->name);
		       flim_plugin_send(plugin, &msg);
		    }
	       }
	  }
     }
   else if (!strcmp(msg->msg, "node_state"))
     {
     }
   else if (!strcmp(msg->msg, "is_toolbar"))
     {
	plugin->is_toolbar = 1;
     }
   else if (!strcmp(msg->msg, "window_id"))
     {
	GdkWindow *window;
	
	window = gdk_window_foreign_new(atoi(msg->dat));
	plugin->windows = g_list_append(plugin->windows, window);
	
	if (plugin->is_toolbar)
	  {
	     GtkWidget *top, *socket, *fixed;
	     
	     top = gtk_widget_get_toplevel(group_list);
	     if (top)
	       {
		  fixed = gtk_object_get_data(GTK_OBJECT(top), "button_area");
		  if (fixed)
		    {
		       socket = gtk_socket_new ();

		       gtk_box_pack_start(GTK_BOX(fixed), socket, FALSE, FALSE, 0);
		       gtk_widget_show(socket);
		       
		       gtk_socket_steal(GTK_SOCKET(socket), atoi(msg->dat));
		       gdk_window_show(window);
		       gdk_flush();
		    }
	       }
	  }
	else
	  {
	     GtkWidget *top, *fixed, *socket;
	     
	     top = gtk_widget_get_toplevel(group_list);
	     if (top)
	       {
		  fixed = gtk_object_get_data(GTK_OBJECT(top), "plugin_area");
		  if (fixed)
		    {
		       int x, y;
		       
		       /* need to figure out a spot */
		       x = 0;
		       y = 0;
		       
		       socket = gtk_socket_new ();		       
		       gtk_box_pack_start(GTK_BOX(fixed), socket, FALSE, FALSE, 2);
		       gtk_widget_show(socket);
		       
		       gtk_socket_steal(GTK_SOCKET(socket), atoi(msg->dat));
		       gdk_window_show(window);
		       gdk_flush();
		    }
	       }
	  }
     }
}

void
flim_plugin_handle_messages(void)
{
   Flim_MsgBuf msgp;
   
   msgp.mtype = 1;
   msgp.mtext[0] = 0;   
   while (msgrcv(msg_id, &msgp, 1024, 0, IPC_NOWAIT) >= 0)
     {
	Msg *msg;
	GList *l;
	
	msg = (Msg *)&(msgp.mtext[0]);
	for (l = plugin_list; l; l = l->next)
	  {
	     Plugin *p;
	     
	     p = l->data;
	     if (p->id == msg->id)
	       {
		  flim_plugin_message(p, msg);
		  break;
	       }
	  }
     }
}

void
flim_plugin_wait(void)
{
   GList *l;
   int all_done = 0;
   
   while (!all_done)
     {
	while (gtk_events_pending()) gtk_main_iteration();
	
	all_done = 1;
	for (l = plugin_list; l; l = l->next)
	  {
	     Plugin *p;
	     
	     p = l->data;
	     if (p->msg_id < 0) all_done = 0;
	  }
	usleep(10000);
     }
}
