/*
 * This program is original work by San Mehat <nettwerk@valinux.com> .
 * This program is distributed under the GNU General Public License (GPL)
 * as outlined in the COPYING file.
 *
 * Copyright (C) 1999, San Mehat, and VA Linux Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <config.h>

#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <dlfcn.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/un.h>
#include <time.h>
#include <errno.h>
#include <module_harness.h>
#include <prototypes.h>

VACM_MODULE_LIST			*mlist_head = NULL;
VACM_MODULE_LIST			*mlist_tail = NULL;

static void			process_loose_module_ipc(VACM_MODULE_LIST *module);
static void			module_destroy(VACM_MODULE_LIST *module);
static int 			send_msg_to_loose_module(VACM_MODULE_LIST *module,
																										 char							*msg);

extern bool_t nexxus_debug;
extern int		errno;
static int		reserved_fds[255];

int	nexxus_api_version_major = 1;
int nexxus_api_version_minor = 1;

int return_num_modules()
{
	VACM_MODULE_LIST	*mlist;
	int								count = 0;

	mlist = mlist_head;
	while(mlist)
		{
		count++;
		mlist = mlist->next;
		}
	return(count);
}

int identify_modules()
{
	struct dirent	**namelist;
	int						n,i;
	int						num_mods = 0;
	char					filename[255];

	for (i = 0; i <255; i++)
		reserved_fds[i] = -1;

	if ((n = scandir(PKGLIBDIR "/modules",&namelist, 0, alphasort))<0)
		{
		nexxus_log_printf(NULL,"Unable to scan modules directory (%m)\n");
		return(0);
		}
	i = 0;
	while(n--)
		{
		if (!strncmp(&namelist[n]->d_name[(strlen(namelist[n]->d_name)-6)],
								 ".loose",
								 6))
			{
			if ((reserved_fds[i] = open("/dev/zero",O_RDONLY))<0)
				printf("[Nexxus] Unable to reserve fd for loose module %s (%m)\n",
							 namelist[n]->d_name);
			i++;
			sprintf(filename,PKGLIBDIR "/modules/%s",namelist[n]->d_name);
			if (!start_loose_module(filename))
				num_mods ++;
			}
		}

	return(num_mods);
}

int start_loose_module(char *filename)
{
	pid_t									pid;

	if (!(pid =fork()))
		{
		execl(filename, filename, "/tmp/vacm", NULL);
		exit(0);
		}
	return(0);
}

static void module_destroy(VACM_MODULE_LIST *module)
{
  if (module == mlist_head)
    {
    mlist_head = module->next;
    if (mlist_head)
      mlist_head->prev = NULL;
    else
      mlist_tail = NULL;
    }
  else if (module == mlist_tail)
    {
    mlist_tail = module->prev;
    if (mlist_tail)
      mlist_tail->next = NULL;
    else
      mlist_head = NULL;
    }
  else
    {
    module->prev->next = module->next;
    module->next->prev = module->prev;
    }
	nettmcs_free(module);
}

int alloc_and_accept_loose_module_connection(int I_fd)
{
	VACM_MODULE_LIST		*mlist_new;
	struct sockaddr_un	addr_un;
	int									len, i;
									
	/* Allocate a new module list entry and link it in */
	if (!(mlist_new = (VACM_MODULE_LIST *) nettmcs_alloc(sizeof(VACM_MODULE_LIST),
																											 "module list entry")))
		{
		nexxus_log_printf(NULL,"Unable to allocate memory for module entry\n");
		return(-1);
		}
	memset(mlist_new, 0, sizeof(VACM_MODULE_LIST));
	if (!mlist_head)
		mlist_head = mlist_tail = mlist_new;
	else
		{
		mlist_tail->next = mlist_new;
		mlist_new->prev = mlist_tail;
		mlist_tail = mlist_new;
		}
	len = sizeof(struct sockaddr_un);

	/* Free up one of the reserved fd's */
	for (i=0;i<255;i++)
		if (reserved_fds[i] != -1)
			break;
	if (i==255)
		printf("[Nexxus] Reserved fd's for loose module connections exhausted\n");
	else
		{
		if (nexxus_debug)
			{
			nexxus_log_printf(
						NULL,
						"Freeing up reserved fd %d for use by loose module(s %d)\n",
						reserved_fds[i],
						i);
			}
		close(reserved_fds[i]);
		reserved_fds[i]= -1;
		}

	if ((mlist_new->loose_fd = accept(I_fd,
															 			(struct sockaddr *) &addr_un,
															 			&len))<0)
		{
		nexxus_log_printf(NULL,"Unable to accept loose module connection (%m)\n");
		return(-1);
		}
	if (nexxus_debug)
		nexxus_log_printf(NULL,"Loose module connection accepted (fd %d)\n",
					 mlist_new->loose_fd);
	client_fd_lock_create(mlist_new->loose_fd);
	mlist_new->state = MOD_STATE_INITIALIZING;
	return(0);
}

void add_loose_modules(int *max, fd_set *read_fds)
{
	VACM_MODULE_LIST	*mlist;

	mlist = mlist_head;
	while(mlist)
		{
		if (mlist->loose_fd > *max)
			*max = mlist->loose_fd;
		FD_SET(mlist->loose_fd, read_fds);
		mlist = mlist->next;
		}
}

void dispatch_loose_module_ipc(fd_set *read_fds)
{
	VACM_MODULE_LIST	*mlist;

	mlist = mlist_head;
	while(mlist)
		{
		if (FD_ISSET(mlist->loose_fd, read_fds))
			process_loose_module_ipc(mlist);
		mlist = mlist->next;
		}

	mlist = mlist_head;
	while(mlist)
		{
		if (mlist->loose_fd == -1)
			{
	  	if (mlist == mlist_head)
 	   		{
 	 	  	mlist_head = mlist->next;
 	  	 	if (mlist_head)
   	   		mlist_head->prev = NULL;
   	 		else
   	   		mlist_tail = NULL;
   	 		} 
	  	else if (mlist == mlist_tail)
 	   		{
 	   		mlist_tail = mlist->prev;
 	   		if (mlist_tail)
 	     		mlist_tail->next = NULL;
 	   		else
 	     		mlist_head = NULL;
 	   		} 
 	 		else
 	   		{
 	   		mlist->prev->next = mlist->next;
 	   		mlist->next->prev = mlist->prev;
 	   		}
			nettmcs_free(mlist);
			mlist = mlist_head;
			if (!mlist)
				break;
			continue;
			}
		mlist = mlist->next;
		}
}

// FMT = JOB_ID:CLIENT_FD:CMD:ARGS...
static void process_loose_module_ipc(VACM_MODULE_LIST *module)
{
	VACM_MODULE_LIST	*module_scan;
	int								client_fd, rc;
	char							*saved_msg = NULL, *last = (char *) 0x00;
	char							*msg = NULL, *bword, *offset;
	__uint32_t				length, brtr, job_id;

	if (!(msg = (char *) nettmcs_alloc(1024, "loose module message")))
		{
		nexxus_log_printf(NULL,
									"Unable to alloc 1k msg buffer for loose module ipc (%m)\n");
		sleep(2);
		return;
		}
	if (module->state == MOD_STATE_UNLOADING)
		return;
	rc = read(module->loose_fd, &length, sizeof(length));
	length = ntohl(length);
	if ((rc == 0) || (rc <0))
		{
		if (rc == -1)
			nexxus_log_printf(NULL,"Error while reading loose module ipc len (%m)\n");
		else if (rc == 0)
			{
			nexxus_log_printf(NULL,
							"Module %s has disconnected from Nexxus\n",module->ipc_tag);
			module_destroy(module);
			}
		goto out_ok;
		}
	brtr = length;
	offset = msg;
	while(brtr)
		{
		rc = read(module->loose_fd, offset, brtr);
		if ((rc == 0) || (rc <0))
			{
			if (rc == -1)
				nexxus_log_printf(NULL,"Error while reading loose module ipc data (%m)\n");
			else if (rc == 0)
				nexxus_log_printf(NULL,"Loose module hangup detected. (module %s)\n",
							 ((module->short_name[0] != (char) 0x00) ? module->short_name : "unknown"));
			client_fd_lock_destroy(module->loose_fd);
			close(module->loose_fd);
			module_destroy(module);
			goto out_ok;
			}
		brtr -= rc;
		offset += rc;
		}
	if (!(saved_msg = (char *) nettmcs_alloc((strlen(msg)+1),
																					 "saved loose module msg")))
		{
		nexxus_log_printf(NULL,"Unable to nettmcs_alloc msg buffer for loose module ipc (%m)\n");
		goto out_ok;
		}
	memcpy(saved_msg, msg, (strlen(msg)+1));

	if (!(bword = strtok_r(msg, ":",&last)))
		goto out_error;
	job_id = (__uint32_t) strtoul(bword, (char **) NULL, 10);
	
	if (!(bword = strtok_r(NULL, ":",&last)))
		goto out_error;
	client_fd = atoi(bword);

	if (!(bword = strtok_r(NULL, ":",&last)))
		goto out_error;

	if (!(strncasecmp(bword, "REG",3)))
		{
		// FMT: 0:0:REG:<SHORT_NAME>:<LONG_NAME>:<DESC>:<AUTHOR>:<VER_MAJ>:<VER_MIN>:<IPC_TAG>
		if (!(bword = strtok_r(NULL,":",&last)))
			goto out_error;
		strncpy(module->short_name, bword, sizeof(module->short_name));

		if (!(bword = strtok_r(NULL,":",&last)))
			goto out_error;
		strncpy(module->long_name, bword, sizeof(module->long_name));

		if (!(bword = strtok_r(NULL,":",&last)))
			goto out_error;
		strncpy(module->desc, bword, sizeof(module->desc));

		if (!(bword = strtok_r(NULL,":",&last)))
			goto out_error;
		strncpy(module->author, bword, sizeof(module->author));

		if (!(bword = strtok_r(NULL,":",&last)))
			goto out_error;
		module->major_version = atoi(bword);

		if (!(bword = strtok_r(NULL,":",&last)))
			goto out_error;
		module->minor_version = atoi(bword);

		if (!(bword = strtok_r(NULL,":",&last)))
			goto out_error;
		strncpy(module->ipc_tag, bword, sizeof(module->ipc_tag));
		
		module_scan = mlist_head;
		while(module_scan)
			{
			if ((!strcmp(module_scan->ipc_tag, module->ipc_tag)) &&
					(module_scan != module))
				{
				char									SZ_deinit[255];

				nexxus_log_printf(NULL,"Cancelling duplicate loose module registration\n");
				strcpy(SZ_deinit,"0:0:DEINIT");
				if (send_msg_to_loose_module(module, SZ_deinit) < 0)
					{
					nexxus_log_printf(NULL,"Unable to send DEINIT to module '%s'\n",
								 module->short_name);
					}
				close(module->loose_fd);
				module_destroy(module);
				goto out_ok;
				}
			module_scan = module_scan->next;
			}
		nexxus_log_printf(NULL,
													"[%s][%d.%d][%s][%s]\n",
													module->short_name,
													module->major_version,
													module->minor_version,
													module->ipc_tag,
													module->author);
		module->state = MOD_STATE_RUNNING;
		configuration_dispatch_to_loose_module(module->ipc_tag);
		goto out_ok;
		}
	else if (!(strncasecmp(bword, "PID",3)))
		{
		char									*pid;

		if (!(pid = strtok_r(NULL,":",&last)))
			goto out_error;

		module->loose_pid = (pid_t) atoi(pid);
		goto out_ok;
		}
	else if (!(strncasecmp(bword, "FOR",3)))
		{
		char	*forward_start;
		char	SZ_forward[1024];

//		printf("Received forward request <%s>\n",saved_msg);
		// FMT: <JOB_ID>:<CLIENT_FD>:FOR:<MSG...>
		forward_start = &saved_msg[((bword - msg)+(strlen(bword)+1))];
		sprintf(SZ_forward,"%s:%d:%s",module->ipc_tag,job_id,forward_start);
		send_msg_to_client_by_fd(client_fd, SZ_forward, (strlen(SZ_forward)+1));
		goto out_ok;
		}
	else if (!(strncasecmp(bword, "SET", 3)))
		{
		char	*node_id;
		char	*mod;
		char	*variable;
		char	*value;
		int		rc;

		// FMT: 0:0:SET:<NODE_ID>:<MODULE>:<VARIABLE>:<VALUE>
		// Module can be set to 'GLOBAL' for a global variable

		if (!(node_id = strtok_r(NULL,":",&last)))
			goto out_error;
		if (!(mod = strtok_r(NULL,":",&last)))
			goto out_error;
		if (!(variable = strtok_r(NULL,":",&last)))
			goto out_error;

		/* BEGIN San mods 07/12/00 
		 * The 'value' field was being stroked so we were losing
		 * any ':'s that may have been in the data 
		 *
		 * OLD code:
		 * if (!(value = strtok_r(NULL,":",&last)))
		 * 	goto out_error;
		 */
		value = &saved_msg[((variable-msg)+(strlen(variable))+1)];
		/* END San mods 07/12/00 */
			
		/* BEGIN San mods 07/12/00 */
		if (!strcmp(value,"ERASE"))
			value = NULL;
		/* END San mods 07/12/00 */

		if (!strncasecmp(mod, "GLOBAL", 6))
			rc = nexxus_variable_set(node_id, NULL, variable, value);
		else
			rc = nexxus_variable_set(node_id, mod, variable, value);
		if (rc < 0)
			{
			nexxus_log_printf(NULL,"Internal Error! Loose module '%s' tried to set\n",
						 module->short_name);
 			nexxus_log_printf(NULL,"variable '%s' for non existant node '%s'!\n",
						 variable,
						 node_id);
			}
		goto out_ok;
		}
	else if (!(strncasecmp(bword, "LOG", 3)))
		{
		char	*log_msg;
		// FMT: 0:0:LOG:<MSG>

		log_msg = &saved_msg[((bword - msg)+(strlen(bword)+1))];
		nexxus_log_printf(module->short_name, "%s\n",log_msg);
		goto out_ok;
		}
	else if (!(strncasecmp(bword, "GET", 3)))
		{
		char	*node_id;
		char	*mod;
		char	*variable;
		char	tmp[255];
		char	response[255];

		// FMT: 0:0:GET:<NODE_ID>:<MODULE>:<VARIABLE>
		if (!(node_id = strtok_r(NULL,":",&last)))
			goto out_error;
		if (!(mod = strtok_r(NULL,":",&last)))
			goto out_error;
		if (!(variable = strtok_r(NULL,":",&last)))
			goto out_error;
		if (!strncasecmp(mod, "GLOBAL", 6))
			rc = nexxus_variable_get(node_id, NULL, variable, &tmp[0], sizeof(tmp));
		else
			rc = nexxus_variable_get(node_id, mod, variable, &tmp[0], sizeof(tmp));
		if (rc < 0)
			{
			nexxus_log_printf(NULL,"Internal Error! Loose module '%s' tried to get\n",
						 module->short_name);
 			nexxus_log_printf(NULL,"non existant variable '%s'!\n", variable);
			sprintf(response,"0:0:ERROR");
			}
		else
			sprintf(response,"0:0:VAR:%s:%s",variable,tmp);
		send_msg_to_loose_module(module, response);
		goto out_ok;
		}
	else
		goto out_error;

out_error:
	nexxus_log_printf(NULL,"Malformatted message from loose module '%s'\n",
				 ((module->short_name[0] != (char) 0x00) ? module->short_name : "unknown"));
	nexxus_log_printf(NULL,"[%s]\n",saved_msg);
out_ok:
	if (msg)
		nettmcs_free(msg);
	if (saved_msg)
		nettmcs_free(saved_msg);

}

int lookup_ipc_tag(char *key, char *out)
{
	VACM_MODULE_LIST *mlist = mlist_head;

	while(mlist)
		{
		if (!strcasecmp(key, mlist->ipc_tag))
			{
			strcpy(out, mlist->ipc_tag);
			return(1);
			}
		mlist = mlist->next;
		}
	return(0);
}

/*
 * Shutdown a module and unload it
 */
int shutdown_module(char *ipc_tag)
{
	VACM_MODULE_LIST			*mlist = mlist_head;
	char									SZ_finalmsg[255];
	time_t								wait_time;
	int										status, rc = 0;

	while(mlist)
		{
		if (!strcasecmp(mlist->ipc_tag, ipc_tag))
			{
			mlist->state = MOD_STATE_UNLOADING;
			strcpy(SZ_finalmsg,"0:0:DEINIT");
			if (send_msg_to_loose_module(mlist, SZ_finalmsg) <0)
				{
				nexxus_log_printf(NULL,
															"Unable to send DEINIT message\n");
				mlist->state = MOD_STATE_RUNNING;
				return(-1);
				}
			close(mlist->loose_fd);

			/* Wait for up to 5 seconds for process to die */
			wait_time = time(NULL) + 5;
			while(wait_time > time(NULL))
				{
				if ((rc = waitpid(mlist->loose_pid, &status, WNOHANG)))
					break;
				}
			if (!rc)
				{
				nexxus_log_printf(NULL,
															"Module %s child process timeout\n",
															mlist->ipc_tag);
				return(0);
				}
			else
				{
				if (WIFEXITED(status))
					{
					nexxus_log_printf(NULL,
																"Module %s exited normally (code %d)\n",
																mlist->ipc_tag,
																WEXITSTATUS(status));
					}
				else if (WIFSIGNALED(status))
					{
					nexxus_log_printf(NULL,
																"Module %s exited due to signal %d\n",
																mlist->ipc_tag,
																WTERMSIG(status));
					}
				}
			break;
			}
		mlist = mlist->next;
		}
	module_destroy(mlist);
	return(0);
}

/*
 * Severity 1 = send deinit
 * Severity 2 = send INT signals
 * Severity 3 = send KILL signals
 */
void shutdown_modules(int severity)
{
	VACM_MODULE_LIST			*mlist = mlist_head;

  KNOWN_UNUSED_PARAM(severity);

	while(mlist)
		{
		shutdown_module(mlist->ipc_tag);
		mlist = mlist->next;
		}
	return;
}

void notify_modules_of_disconnection(int fd)
{
	VACM_MODULE_LIST	*mlist = mlist_head;
	char							SZ_finalmsg[255];

	while(mlist)
		{
		sprintf(SZ_finalmsg,"0:%d:DISCON",fd);
		if (send_msg_to_loose_module(mlist, SZ_finalmsg) <0)
			{
			mlist = mlist->next;
			continue;
			}
		mlist = mlist->next;
		}
	return;
}

int dispatch_ipc_to_module(int					I_fd,
													 char					*key, 
													 char					*msg,
													 char					*tag,
													 char					*node_id,
													 __uint32_t		job_id)
{
	VACM_MODULE_LIST *mlist = mlist_head;

	while(mlist)
		{
		if (!strcasecmp(key, mlist->ipc_tag))
			{
			char	SZ_finalmsg[1024];

			strcpy(tag, mlist->ipc_tag);
			sprintf(SZ_finalmsg,
							"%d:%d:IPC:%s:%s",
							job_id,
							I_fd,
							node_id,
							msg);
			if (send_msg_to_loose_module(mlist, SZ_finalmsg) < 0)
				return(-EIO);
			return(0);
			}
		mlist = mlist->next;
		}
	return(-ENOENT);
}

char *return_module_state(VACM_MODULE_LIST *mlist)
{
		switch(mlist->state)
			{
			case MOD_STATE_LOADED:
				return strdup("STATE_LOADED");
			case MOD_STATE_INITIALIZING:
				return strdup("STATE_INITIALIZING");
			case MOD_STATE_RUNNING:
				return strdup("STATE_RUNNING");
			case MOD_STATE_STOPPING:
				return strdup("STATE_STOPPING");
			case MOD_STATE_UNLOADING:
				return strdup("STATE_UNKNOWN");
			default:
				return strdup("STATE_UNKNOWN");
			}
}

int dispatch_config_line_to_loose_module(char *ipc_tag, char *module, char *msg)
{
	VACM_MODULE_LIST	*mlist = mlist_head;
	char							*loose_msg;

	KNOWN_UNUSED_PARAM(module);
	// Find the module first
	while(mlist)
		{
		if (!strcmp(ipc_tag, mlist->ipc_tag))
			break;
		mlist = mlist->next;
		}
	if (!mlist)
		return(-1);


	if (!(loose_msg = (char *)nettmcs_alloc(((strlen(msg)+1) + 10), "loose msg")))
		{
		nexxus_log_printf(NULL,"Unable to alloc memory for loose msg\n");
		return(-1);
		} 
	sprintf(loose_msg,"0:0:CFG:%s",msg);
#if 0
	printf("config: sending <%s> to loose module %s (fd %d)\n",
				 loose_msg,
				 mlist->short_name,
				 mlist->loose_fd);
#endif
	if (send_msg_to_loose_module(mlist, loose_msg) < 0)
		{
		nettmcs_free(loose_msg);
		return(-1);
		}

	nettmcs_free(loose_msg);
	return(0);
}

int dispatch_config_line(char *module, char *msg)
{
	VACM_MODULE_LIST	*mlist = mlist_head;
	char							tmp_msg[255];
	char							loose_msg[255];

	if (!module)
		{
		/* Global */
		while (mlist)
			{
			strcpy(tmp_msg, msg);
			if (nexxus_debug)
				nexxus_log_printf(NULL,
															"Dispatching global configuration line '%s'\n",
															msg);
			sprintf(loose_msg,"0:0:CFG:%s",msg);
			if (send_msg_to_loose_module(mlist, loose_msg) < 0)
				{
				mlist = mlist->next;
				continue;
				}
			mlist = mlist->next;
			}
		}
	else
		{
		while (mlist)
			{
			if (!strcasecmp(mlist->ipc_tag, module))
				{
				char loose_msg[255];

				if (nexxus_debug)
					nexxus_log_printf(NULL,
																"Dispatching %s configuration line '%s'\n",
																module,
																msg);
				sprintf(loose_msg,"0:0:CFG:%s",msg);
				if (send_msg_to_loose_module(mlist, loose_msg) < 0)
					return(0);
				}
			mlist = mlist->next;
			}
		return(-1);
		}
	return(0);
}

static int send_msg_to_loose_module(VACM_MODULE_LIST *module, char *msg)
{
	__uint32_t	length;
	__uint32_t	net_length;
	int					rc;

	nexxus_do_lock(module->loose_fd);
	length = (strlen(msg)+1);
	net_length = htonl(length);

	if (nexxus_debug)
		{
		printf("sending msg <%s> to module %s (fd %d)\n",
					 msg,
					 module->ipc_tag,
					 module->loose_fd);
		}
	rc = write(module->loose_fd, &net_length, sizeof(net_length));
	if (rc < 0)
		{
		if (errno != EPIPE)
			{
			nexxus_log_printf(NULL,"Unable to send msg to loose module '%s' (%m)\n",
				 	 	module->short_name);
			nexxus_log_printf(NULL,"MSG <%s>\n",msg);
			}
		nexxus_do_unlock(module->loose_fd);
		return(-1);
		}
	if (write(module->loose_fd, msg,length) < 0)
		{
		if (errno != EPIPE)
			{
			nexxus_log_printf(NULL,"Unable to send msg to loose module '%s' (%m)\n",
				 	 	module->short_name);
			nexxus_log_printf(NULL,"MSG <%s>\n",msg);
			}
		nexxus_do_unlock(module->loose_fd);
		return(-1);
		}
	nexxus_do_unlock(module->loose_fd);
	if (nexxus_debug)
		printf("msg sent to loose module\n");
	return(0);
}

void handle_child_exit(pid_t	pid, int status)
{
	VACM_MODULE_LIST	*mlist = mlist_head;
	char							log_msg[255];
	char							tmp_msg[255];

	while(mlist)
		{
		if (mlist->loose_pid == pid)
			break;
		mlist = mlist->next;
		}
	if (mlist)
		sprintf(log_msg,"Module %s (pid %d)",mlist->ipc_tag,pid);
	else
		sprintf(log_msg,"Child process (pid %d)",pid);

	if (WIFEXITED(status))
		{
		sprintf(tmp_msg," has exited normally (code %d)\n",WEXITSTATUS(status));
		strcat(log_msg,tmp_msg);
		}
	else if (WIFSIGNALED(status))
		{
		sprintf(tmp_msg," has exited due to signal %d\n",WTERMSIG(status));
		strcat(log_msg,tmp_msg);
		}
	else
		{
		sprintf(tmp_msg," has exited for unknown reason\n");
		strcat(log_msg,tmp_msg);
		}
	nexxus_log_printf(NULL,log_msg);
	if (mlist)
		{
		client_fd_lock_destroy(mlist->loose_fd);
		close(mlist->loose_fd);
		module_destroy(mlist);
		}
}

