/*
 * This program is original work by San Mehat <nettwerk@valinux.com> .
 * This program is distributed under the GNU General Public License (GPL)
 * as outlined in the COPYING file.
 *
 * Copyright (C) 1999, San Mehat, and VA Linux Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Notes: This is now a stand-alone run-time linkable module for VACM
 * Symbols beginning with 'api' denote a module harness required
 * symbol which should not be renamed. Renaming an 'api' symbol
 * will cause undesired operation, and program failure.
 *
 */

#ifndef _EMP_H
#define _EMP_H

#define EMP_ACTIVATE 0x1b4f39 /* When in console mode, escape stream to EMP controller */

/*
	Frame bytes
 */
#define EMP_START_CHAR       0xa0
#define EMP_STOP_CHAR        0xa5
#define EMP_HANDSHAKE_CHAR   0xa8
#define EMP_DATA_ESCAPE_CHAR 0xaa

/*
	Encoded data escape sequences
 */
#define EMP_DATA_ESCAPE_B0   0xa0
#define EMP_DATA_ESCAPE_B5   0xa5
#define EMP_DATA_ESCAPE_BA   0xaa
#define EMP_DATA_ESCAPE_B8   0xa8
#define EMP_DATA_ESCAPE_B6   0xa6

#define EMP_MAX_MSG_LEN 32 /* Maximum EMP message length */

#define CMD_GET_CHASSIS_CAPABILITIES      0x00
#define CMD_GET_CHASSIS_STATUS            0x01
#define CMD_CHASSIS_CONTROL               0x02
#define CMD_IDENTIFY_CHASSIS              0x04
#define CMD_GET_FRU_INV_AREA_INFO         0x10
#define CMD_READ_FRU_INV_DATA             0x11
#define CMD_WRITE_FRU_INV_DATA            0x12
#define CMD_GET_SDR_REPOSITORY_INFO       0x20
#define CMD_GET_SDR_REPOSITORY_ALLOC_INFO 0x21
#define CMD_RESERVE_SDR_REPOSITORY        0x22
#define CMD_GET_SDR                       0x23
#define CMD_GET_SDR_REPOSITORY_TIME       0x28
#define CMD_GET_SENSOR_READING						0x2D
#define CMD_GET_SEL_INFO                  0x40
#define CMD_GET_SEL_ALLOC_INFO            0x41
#define CMD_RESERVE_SEL                   0x42
#define CMD_GET_SEL_ENTRY                 0x43
#define	CMD_DEL_SEL_ENTRY									0x46
#define CMD_CLEAR_SEL                     0x47
#define CMD_GET_SEL_TIME                  0x48
#define CMD_GET_EMP_CONFIGURATION         0x71
#define CMD_SUBMIT_PASSWORD               0x72
#define CMD_SET_EMP_REINIT                0x75
#define CMD_EMP_ACTIVE                    0x78
#define CMD_REARM_EMP_CONNECTION_TIMER    0x7b
#define CMD_SET_BIOS_FLAGS                0x7c
#define CMD_GET_BIOS_FLAGS                0x7d
#define CMD_GET_DEVICE_ID									0x01

/* CHASSIS_CONTROL args */
#define CHASSIS_CONTROL_PWROFF   0x00
#define CHASSIS_CONTROL_PWRON    0x01
#define CHASSIS_CONTROL_PWRCYCLE 0x02
#define CHASSIS_CONTROL_RESET    0x03
#define CHASSIS_CONTROL_PULSE_FP 0x04

/* GET_EMP_CONFIGURATION BITS */
typedef struct emp_config_resp
	{
	unsigned char	cc;

#define	PRE_BOOT_ONLY		0x00
#define	ALWAYS_ACTIVE		0x01
#define	ALWAYS_DISABLED	0x02

	unsigned char	activation_mode:2;
	unsigned char	restricted_mode_enabled:1;
	unsigned char	direct_connect_mode:1;
	unsigned char	dcd_activation_enabled:1;
	unsigned char	password_null:1;
	unsigned char	reserved:2;

	unsigned char	emp_busy:1;
	unsigned char	emp_switch:1;
	unsigned char	skip_reinit:1;
	unsigned char	console_switch_request:1;
	unsigned char	res1:1;
	unsigned char	res2:3;
	} EMP_CONFIG_RESP;

/* GET_CHASSIS_CAPABILITIES BITS */
typedef struct capable_response
{
  unsigned char cc;

	unsigned char provides_intrusion:1;
	unsigned char provides_secure:1;
	unsigned char provides_fp_nmi:1;
	unsigned char provides_power_interlock:1;
	unsigned char res:4;

  unsigned char cflag;
  unsigned char fru_addr;
  unsigned char sdr_addr;
  unsigned char sel_addr;
  unsigned char sm_addr;
  unsigned char icmb_addr;
  } CAPABILITIES_RESPONSE;

#define CAPABLE_INTRUSION       0x01
#define CAPABLE_SECURE          0x02
#define CAPABLE_FP_NMI          0x04
#define CAPABLE_POWER_INTERLOCK 0x08

/* GET_DEVICE_ID RESPONSE */
typedef struct device_id_response
	{
	unsigned char cc;
	unsigned char device_id;

	unsigned char device_revision					:4;
	unsigned char reserved								:3;
	unsigned char provides_sdr						:1;

	unsigned char	major_firmware_revision	:7;
	#define NORMAL_OPERATION	0
	#define DEVICE_BUSY				1
	unsigned char device_available				:1;

	unsigned char	minor_firmware_revision;

	unsigned char	ipmi_version_major			:4;
	unsigned char	ipmi_version_minor			:4;

	unsigned char supports_sensor_device	:1;
	unsigned char supports_sdr_device			:1;
	unsigned char supports_sel_device			:1;
	unsigned char supports_fru_device			:1;
	unsigned char supports_ipmb_receiver	:1;
	unsigned char supports_ipmb_generator	:1;
	unsigned char supports_bridge					:1;
	unsigned char	supports_chassis_device	:1;

	unsigned char manufacturer_id1;
	unsigned char	manufacturer_id2;
	unsigned char	manufacturer_id3;

	unsigned short product_id;
	} DEVICE_ID_RESPONSE;

/* GET_CHASSIS_STATUS BITS */
typedef struct chassis_status_response
	{
	unsigned char cc;

	unsigned char ps_power_on								:1;
	unsigned char	ps_power_overload					:1;
	unsigned char	ps_interlock_active				:1;
	unsigned char	ps_power_fault						:1;
	unsigned char ps_power_control_fault		:1;
	unsigned char	ps_power_restore_policy		:1;
	unsigned char	ps_reserved								:2;

	unsigned char	lpe_ac_failed							:1;
	unsigned char	lpe_power_overload				:1;
	unsigned char	lpe_interlock_power_down	:1;
	unsigned char	lpe_power_fault						:1;
	unsigned char	lpe_commanded_onoff				:1;
	unsigned char	lpe_reserved							:3;

	unsigned char	mcs_intrusion_active			:1;
	unsigned char	mcs_secure_mode_active		:1;
	unsigned char	mcs_drive_fault						:1;
	unsigned char	mcs_cooling_fan_fault			:1;
	unsigned char	mcs_reserved							:4;
	} CHASSIS_STATUS_RESPONSE;

/* 
 * NOTE: These structures only detail the beginning of 
 *       the data packets... (framing is also not included) 
 */
typedef struct reqhdr
	{

#define EMP_SLAVE_ADDRESS 0x20
	unsigned char rs_slave_addr;

	unsigned char lun:2;

	/*
	 * The network function codes
	 */
#define CHASSIS_REQUEST		0x00
#define CHASSIS_RESPONSE	0x01
#define BRIDGE_REQUEST		0x02
#define BRIDGE_RESPONSE		0x03
#define SENSOR_REQUEST		0x04
#define SENSOR_RESPONSE		0x05
#define APP_REQUEST				0x06
#define APP_RESPONSE			0x07
#define	FIRMWARE_REQUEST	0x08
#define FIRMWARE_RESPONSE	0x09
#define STORAGE_REQUEST		0x0A
#define STORAGE_RESPONSE	0x0B
	unsigned char net_fn:6;

	unsigned char cksum1;

#define EMP_APPLICATION_SLAVE_ADDRESS 0xC4
	unsigned char rq_slave_addr;

	unsigned char rqseq;
	unsigned char cmd;
	}REQHDR; /* After this there are x data bytes, then a checksum */

typedef struct resphdr
	{
	unsigned char rq_slave_addr;
	unsigned char net_fn;
	unsigned char cksum1;
	unsigned char rs_slave_addr;
	unsigned char rqseq;
	unsigned char cmd;
	unsigned char completion_code;
	} RESPHDR; /* After this there are x bytes of response data,then a checksum */

/* FRU defines */
typedef struct fru_inv_area_info
	{
	unsigned char		cc   __attribute__ ((packed));
	unsigned short	size __attribute__ ((packed));

#define ACCESSMODE_BYTE	0
#define ACCESSMODE_WORD	1
	unsigned char		accessmode:1;
	unsigned char		reserved:7;
	} FRU_INV_AREA_INFO;

typedef struct fru_inventory_data_request_ipmi09
	{
	unsigned short offset __attribute__ ((packed));
	unsigned char  count  __attribute__ ((packed));
	} FRU_INV_DATA_REQ_IPMI09;

typedef struct fru_inventory_data_request_ipmi10
	{
	unsigned char		dev_id	__attribute__ ((packed));
	unsigned short	offset	__attribute__ ((packed));
	unsigned char		count		__attribute__ ((packed));
	} FRU_INV_DATA_REQ_IPMI10;

typedef struct fru_common_header
	{
	unsigned char common_header_format_version;
	unsigned char internal_use_offset;
	unsigned char	chassis_info_offset;
	unsigned char board_area_offset;
	unsigned char product_info_offset;
	unsigned char multirecord_area_offset;
	unsigned char pad;
	unsigned char checksum;
	} FRU_COMMON_HEADER;

/* Something to supress unused param warnings */
#define KNOWN_UNUSED_PARAM(x) { int __bleh; __bleh = (int) x;}
#endif
