/*
 * This program is original work by San Mehat <nettwerk@valinux.com> .
 * This program is distributed under the GNU General Public License (GPL)
 * as outlined in the COPYING file.
 *
 * Copyright (C) 1999, San Mehat, and VA Linux Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 *  with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef _EMP_SLAVE_CONTEXT_H
#define _EMP_SLAVE_CONTEXT_H

#include <config.h>
#include <pthread.h>
#include <emp.h>
#include <emp_sdr.h>

struct fru_data
	{
	char	*board_manufacturer;
	char	*board_product_name;
	char	*board_serial;
	char	*board_part_number;
	char	*product_manuf;
	char	*product_name;
	char	*product_part;
	char	*product_ver;
	char	*product_serial;
	char	*product_asset;
	int		fru_at_offset;
	int		fru_at_reserved_bytes;
	};

struct versions
	{
	int	ipmi_ver_major;
	int	ipmi_ver_minor;
	int	bmc_ver_major;
	int	bmc_ver_minor;

	int	device_revision;
	int	manufacturer_id;
	};

struct capable
	{
	int	has_sensor;
	int	has_sdr;
	int	has_sel;
	int has_fru;
	int	has_ipmb_recv;
	int has_ipmb_gen;
	int	has_bridge;
	int	has_chassis;

	int has_intrusion;
	int has_fpnmi;
	int has_power_interlock;
	};

struct icmb_addrs
	{
	unsigned char	fru_devid;
	unsigned char	fru_addr;
	unsigned char	sdr_addr;
	unsigned char	sel_addr;
	unsigned char	sm_addr;
	unsigned char	icmb_addr;
	};
struct emp_bios_cfg
	{
	int	activation_mode;
	int	restricted_mode;
	int	direct_connect;
	int	dcd_activation;
	int	password_null;
	};

#define MAX_SEL_WATCHERS	255

typedef struct context
	{
	pthread_t						thread;
	pthread_attr_t			attr;

	pthread_t						driver_thread;
	pthread_attr_t			driver_attr;

	pthread_t						sensor_thread;
	pthread_attr_t			sensor_attr;

	int									emp_sock;

	char								*node_id;
	char								*device_address;
	char								*password;
	char								*ip_address;

	int									thread_ipc_read;
	int									thread_ipc_write;
	pthread_mutex_t			ipc_writelock;

	int									slave_ipc_read;
	int									slave_ipc_write;

	int									sensor_ipc_write;
	int									sensor_ipc_read;

	int									driver_pwrite;
	int									thread_pread;
	int									driver_pread;
	int									thread_pwrite;
	unsigned char				sequencenumber;
	int									console_switch_allow; 
	int									console_mode;
	int									negotiation_complete;

	int									protocol_detected;
	int									enable_driver_checks;
	time_t							last_active_timestamp;
	int									session_active;

	time_t							next_seldl;
	time_t							next_rearm;
	time_t							next_cstatus;

	unsigned char				*fru_buffer;
	struct fru_data			fru_data;
	struct versions			ver;
	struct capable			capabilities;
	struct icmb_addrs		addrs;
	struct emp_bios_cfg	bios_cfg;
	SENSOR							*sensor_list;

	int									sel_watchers[MAX_SEL_WATCHERS];

	// FD for sending out extentd requests
	int									emp_extentd_fd;
	pthread_mutex_t			extentd_lock;
	int									extentd_connect_status;

	int									refcount;
	struct context			*next;
	struct context			*prev;
	} CONTEXT;

typedef struct driver_ipc
	{
#define DIPCREQ_TRANS	1 // Thread -> Driver do transaction request
#define DIPCREQ_RESP	2 // Driver -> Thread response data for transaction
#define DIPCREQ_NEG		3	// Driver -> Thread request negotiation
#define DIPCREQ_CB		4 // Driver -> Thread console byte
#define DIPCREQ_CON		5 // Driver -> Thread request submit pass
#define DIPCREQ_DET		6 // Thread -> Driver request protocol detect
												// Driver -> Thread reply to protocol detect

	char					request;
	int						rc;			// return code
	unsigned char	byte;		// console_byte
	int						pkt_len;
	unsigned char	cmd;
	unsigned char	pkt_buffer[EMP_MAX_MSG_LEN *2];	// frame
	} DRIVER_IPC;
#endif
