/*
 * This program is original work by San Mehat <nettwerk@valinux.com> .
 * This program is distributed under the GNU General Public License (GPL)
 * as outlined in the COPYING file.
 *
 * Copyright (C) 1999, San Mehat, and VA Linux Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * VACM Remote System Status Daemon
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <sys/resource.h>
#include <crypt.h>
#include <vacm_sys_stat_proxy.h>
#include <config.h>

#ifdef OPENSSL
#include <openssl/rsa.h>
#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

#define SSL_HOME "/etc"
#define CERTF SSL_HOME "/vacm.cert"
#define KEYF SSL_HOME "/vacm.key"
#endif

static void process_cmd(NEXXUS	*nexxusp, char *buffer);
static void send_msg(int fd, char *msg);
static int	get_password(void);
static int	set_password(void);
static void become_daemon(void);
static int	do_connect(NEXXUS	*nexxusp);

#ifdef OPENSSL
static SSL *lookup_nexxus_ssl_by_fd(int fd);
#endif

#define TIMER_INTERVAL 1
#define GRACE_PERIOD   0

static int				comm_sock;
static NEXXUS			nexxus[255];
int								debug =0;
char							daemon_password[255];
int								foreground_mode = 0;

#ifdef OPENSSL
static SSL_CTX		*server_ctx = NULL;
static SSL_CTX		*client_ctx = NULL;
static SSL_METHOD	*meth = NULL;
#endif

char	passwdpath[255];

int main(argc, argv)
int		argc;
char	**argv;
{
	struct sockaddr_in	sock;
	int									I_optval = 1;
	fd_set							read_fds;
	struct timeval			to;
	int									max, I_c1, len, rc;
	char								buffer[512];
	char								keypath[255];
#ifdef OPENSSL
	int									err;
#endif

#if 1
	if (getuid() != 0)
		{
		printf("You must be root to run this\n");
		exit(-1);
		}
#endif

	printf("VACM System Status Daemon Proxy version 1.0 (nettwerk@valinux.com)\n");
	memset(&keypath[0], 0, sizeof(keypath));
	memset(&passwdpath[0], 0, sizeof(passwdpath));
	while ((rc = getopt(argc, argv, "pfk:K:"))!=EOF)
		{
		if ((char)rc == 'p')
			{
			if (!set_password())
				exit(-1);
			exit(0);
			}
		else if ((char)rc =='f')
			foreground_mode = 1;
		else if ((char)rc =='k')
			strncpy(keypath,optarg,(sizeof(keypath)-1));
		else if ((char)rc =='K')
			strncpy(passwdpath,optarg,(sizeof(passwdpath)-1));
		else
			{
			printf("Usage: %s [-p] [-f] [-k keyfile] [-K passwdfile]\n",
						argv[0]);
			printf("       [-p] flag changes the nexxus access password\n");
			printf("       [-f] flag sets daemon to run in foreground\n");
			printf("       [-k] specify a different keyfile\n");
			printf("       [-K] specify a different passwdfile\n");
			exit(-1);
			}
		}
		
	if (!get_password())
		{
		printf("No nexxus access password set. Please set it\n");
		if (!set_password())
			{
			printf("Unable to set password.\n");
			exit(-1);
			}
		printf("Password set. Please rerun proxy daemon\n");
		exit(0);
		}

	if (!foreground_mode)
		become_daemon();

	signal(SIGPIPE, SIG_IGN);
#ifdef OPENSSL
  SSL_load_error_strings();
  SSLeay_add_ssl_algorithms();
  meth = SSLv23_server_method();
  server_ctx = SSL_CTX_new(meth);
  if (!server_ctx)
    {
    ERR_print_errors_fp(stderr);
		exit(-1);
    }
	if (keypath[0]=='\0')
		{
  	if (SSL_CTX_use_certificate_file(server_ctx, CERTF, SSL_FILETYPE_PEM) <=0)
    	{
    	ERR_print_errors_fp(stderr);
			exit(-1);
    	}
  	if (SSL_CTX_use_PrivateKey_file(server_ctx, KEYF, SSL_FILETYPE_PEM) < 0)
    	{
    	ERR_print_errors_fp(stderr);
			exit(-1);
    	}
		}
	else
		{
		printf("Using user specified keyfile %s\n",keypath);
  	if (SSL_CTX_use_certificate_file(server_ctx, keypath, SSL_FILETYPE_PEM) <=0)
    	{
    	ERR_print_errors_fp(stderr);
			exit(-1);
    	}
  	if (SSL_CTX_use_PrivateKey_file(server_ctx, keypath, SSL_FILETYPE_PEM) < 0)
    	{
    	ERR_print_errors_fp(stderr);
			exit(-1);
    	}
		}
  if (!SSL_CTX_check_private_key(server_ctx))
    {
    printf("Private key does not match the certificate public key\n");
		exit(-1);
    }
  meth = SSLv23_client_method();
  client_ctx = SSL_CTX_new(meth);
  if (!client_ctx)
    {
    ERR_print_errors_fp(stderr);
		exit(-1);
    }
	printf("OpenSSL encryption layer enabled\n");
#endif
	if ((comm_sock = socket(AF_INET, SOCK_STREAM, 0))<0)
		{
		perror("socket");
		exit(1);
		}
	if (setsockopt(comm_sock,
								 SOL_SOCKET,
								 SO_REUSEADDR,
								 &I_optval,
								 sizeof(I_optval)) < 0)
		{
		perror("setsockopt");
		exit(1);
		}
	memset(&sock, 0, sizeof(sock));
	sock.sin_family = AF_INET;
	sock.sin_addr.s_addr = htonl(INADDR_ANY);
	sock.sin_port = htons(NETWORK_PORT);

	if (bind(comm_sock,
					 (struct sockaddr *) &sock,
					 sizeof(sock))<0)
		{
		perror("bind");
		exit(1);
		}
	if (listen(comm_sock, 5)<0)
		{
		perror("listen");
		exit(1);
		}
	for (I_c1=0; I_c1<255;I_c1++)
		{
		nexxus[I_c1].out_sock = -1;
		nexxus[I_c1].in_sock = -1;
#ifdef OPENSSL
		nexxus[I_c1].in_ssl = NULL;
		nexxus[I_c1].out_ssl = NULL;
#endif
		}

	while(1)
		{
		FD_ZERO(&read_fds);
		to.tv_sec = 10;
		to.tv_usec = 0;
		FD_SET(comm_sock, &read_fds);
		max = comm_sock;

		for (I_c1 = 0; I_c1 < 255; I_c1++)
			{
			if (nexxus[I_c1].in_sock != -1)
				{
				if (max < nexxus[I_c1].in_sock)
					max = nexxus[I_c1].in_sock;
				FD_SET(nexxus[I_c1].in_sock, &read_fds);
				}
			if (nexxus[I_c1].out_sock != -1)
				{
				if (max < nexxus[I_c1].out_sock)
					max = nexxus[I_c1].out_sock;
				FD_SET(nexxus[I_c1].out_sock, &read_fds);
				}
			}
		if ((rc = select((max+1), &read_fds, NULL, NULL, &to))<0)
			{
			if (errno == EINTR)
				continue;
			perror("select");
			continue;
			}
		else if (!rc)
			{
			continue;
			}
		else 
			{
			if (FD_ISSET(comm_sock, &read_fds))
				{
				/* New client connection */
				for(I_c1 =0; I_c1<255;I_c1++)
					if (nexxus[I_c1].in_sock == -1)
						break;
				if (I_c1 == 255)
					{
					printf("[VACM_SYS_STAT_PROXY] Maximum number of Nexxus's connected\n");
					sleep(1);
					continue;
					}
				len = sizeof(struct sockaddr_in);
				if ((nexxus[I_c1].in_sock = accept(comm_sock,
																			 (struct sockaddr *) &sock,
																			 &len))<0)
					{
					perror("accept");
					sleep(1);
					continue;
					}
				nexxus[I_c1].out_sock = -1;
				if (debug)
					{
					printf("[VACM_SYS_STAT_PROXY] Nexxus connection accepted (fd %d)\n",
								 nexxus[I_c1].in_sock);
					}
#ifdef OPENSSL
				nexxus[I_c1].in_ssl = SSL_new(server_ctx);
				SSL_set_fd(nexxus[I_c1].in_ssl, nexxus[I_c1].in_sock);
				err = SSL_accept(nexxus[I_c1].in_ssl);
    		if (!strcmp(SSL_get_cipher(nexxus[I_c1].in_ssl), "(NONE)"))
      		{
          printf("Non-SSL client detected. Disconnecting.\n");
      		SSL_free(nexxus[I_c1].in_ssl);
					close(nexxus[I_c1].in_sock);
					close(nexxus[I_c1].out_sock);
					nexxus[I_c1].authenticated = 0;
					nexxus[I_c1].in_sock = -1;
					nexxus[I_c1].out_sock = -1;
					continue;
      		}
				if (debug)
					printf("SSL connection using %s\n",SSL_get_cipher(nexxus[I_c1].in_ssl));
				if (!(nexxus[I_c1].cert = SSL_get_peer_certificate(nexxus[I_c1].in_ssl)))
					{
					if (debug)
						printf("Note: Client does not have a certificate\n");
					}
#endif
				} /* end new client connection */
			for (I_c1 =0; I_c1<255;I_c1++)
				{
				if (nexxus[I_c1].in_sock != -1)
					{
					if (FD_ISSET(nexxus[I_c1].in_sock, &read_fds))
						{
						__uint32_t	len;
		
						if (debug)
							{
							printf("[VACM_SYS_STAT_PROXY] Data ready on fd %d\n",
										 nexxus[I_c1].in_sock);
							}
#ifdef OPENSSL
						rc = SSL_read(nexxus[I_c1].in_ssl, (char *) &len, sizeof(len));
						if ((!rc) || (rc <0))
							{
							SSL_shutdown(nexxus[I_c1].in_ssl);
							SSL_free(nexxus[I_c1].in_ssl);
							close(nexxus[I_c1].in_sock);
							close(nexxus[I_c1].out_sock);
							nexxus[I_c1].authenticated = 0;
							nexxus[I_c1].in_sock = -1;
							nexxus[I_c1].out_sock = -1;
							continue;
							}
#else
						rc = read(nexxus[I_c1].in_sock, (char *) &len, sizeof(len));
						if (rc <= 0)
							{
							if (rc < 0)
								perror("read 1");
							if (debug)
								{
								printf("[VACM_SYS_STAT_PROXY] Closing Nexxus fd %d\n",
										 	 nexxus[I_c1].in_sock);
								}
							close(nexxus[I_c1].in_sock);
							close(nexxus[I_c1].out_sock);
							nexxus[I_c1].authenticated = 0;
							nexxus[I_c1].in_sock = -1;
							nexxus[I_c1].out_sock = -1;
							continue;
							}
#endif
						len = ntohl(len);
#ifdef OPENSSL
						rc = SSL_read(nexxus[I_c1].in_ssl, (char *) buffer, len);
						if ((!rc) || (rc <0))
							{
							SSL_shutdown(nexxus[I_c1].in_ssl);
							SSL_free(nexxus[I_c1].in_ssl);
							if (nexxus[I_c1].out_ssl)
								SSL_free(nexxus[I_c1].out_ssl);
							close(nexxus[I_c1].in_sock);
							close(nexxus[I_c1].out_sock);
							nexxus[I_c1].authenticated = 0;
							nexxus[I_c1].in_sock = -1;
							nexxus[I_c1].out_sock = -1;
							continue;
							}
#else
						rc = read(nexxus[I_c1].in_sock, buffer, len);
						if (rc <= 0)
							{
							if (rc < 0)
								perror("read 2");
							printf("[VACM_SYS_STAT_PROXY] Closing Nexxus fd %d\n",
										 nexxus[I_c1].in_sock);
							close(nexxus[I_c1].in_sock);
							close(nexxus[I_c1].out_sock);
							nexxus[I_c1].authenticated = 0;
							nexxus[I_c1].in_sock = -1;
							nexxus[I_c1].out_sock = -1;
							continue;
							}
#endif
						process_cmd(&nexxus[I_c1], buffer);
						}
					} /* end if in_sock valid */
				if (nexxus[I_c1].out_sock != -1)
					{
					if (FD_ISSET(nexxus[I_c1].out_sock, &read_fds))
						{
						__uint32_t	len;
		
						if (debug)
							{
							printf("[VACM_SYS_STAT_PROXY] Data ready on fd %d\n",
										 nexxus[I_c1].out_sock);
							}
#ifdef OPENSSL
						rc = SSL_read(nexxus[I_c1].in_ssl, (char *) &len, sizeof(len));
						if ((!rc) || (rc <0))
							{
							SSL_shutdown(nexxus[I_c1].in_ssl);
							SSL_free(nexxus[I_c1].in_ssl);
							close(nexxus[I_c1].in_sock);
							close(nexxus[I_c1].out_sock);
							nexxus[I_c1].authenticated = 0;
							nexxus[I_c1].in_sock = -1;
							nexxus[I_c1].out_sock = -1;
							continue;
							}
#else
						rc = read(nexxus[I_c1].out_sock, (char *) &len, sizeof(len));
						if (rc <= 0)
							{
							if (rc < 0)
								perror("read 1");
							if (debug)
								{
								printf("[VACM_SYS_STAT_PROXY] Closing Nexxus fd %d\n",
										 	 nexxus[I_c1].out_sock);
								}
							close(nexxus[I_c1].in_sock);
							close(nexxus[I_c1].out_sock);
							nexxus[I_c1].authenticated = 0;
							nexxus[I_c1].in_sock = -1;
							nexxus[I_c1].out_sock = -1;
							continue;
							}
#endif
						len = ntohl(len);
#ifdef OPENSSL
						rc = SSL_read(nexxus[I_c1].in_ssl, (char *) buffer, len);
						if ((!rc) || (rc <0))
							{
							SSL_shutdown(nexxus[I_c1].in_ssl);
							SSL_free(nexxus[I_c1].in_ssl);
							if (nexxus[I_c1].out_ssl)
								SSL_free(nexxus[I_c1].out_ssl);
							close(nexxus[I_c1].in_sock);
							close(nexxus[I_c1].out_sock);
							nexxus[I_c1].authenticated = 0;
							nexxus[I_c1].in_sock = -1;
							nexxus[I_c1].out_sock = -1;
							continue;
							}
#else
						rc = read(nexxus[I_c1].out_sock, buffer, len);
						if (rc <= 0)
							{
							if (rc < 0)
								perror("read 2");
							printf("[VACM_SYS_STAT_PROXY] Closing Nexxus fd %d\n",
										 nexxus[I_c1].out_sock);
							close(nexxus[I_c1].in_sock);
							close(nexxus[I_c1].out_sock);
							nexxus[I_c1].authenticated = 0;
							nexxus[I_c1].in_sock = -1;
							nexxus[I_c1].out_sock = -1;
							continue;
							}
#endif
						/* Forward this off to nexxus */
						send_msg(nexxus[I_c1].in_sock,buffer);
						}
					} /* end if out_sock valid */
				} /* end for */
			} /* end else */
		} /* end while */
}

static void process_cmd(NEXXUS *nexxusp, char *buffer)
{
	char	*type;
	int		fd;
	char	saved_msg[512];

	strncpy(saved_msg, buffer, (sizeof(saved_msg)-1));

	fd = nexxusp->in_sock;
	if (debug)
		printf("[VACM_SYS_STAT_PROXY] Processing <%s>\n",buffer);
	if (!(type = strtok(buffer,":")))
		return;
	if (!nexxusp->authenticated)
		{
		if (!strcmp(type,"PROXY_AUTH"))
			{
			char	*password;
			char	*crypted;
			if (!(password = strtok(NULL,":")))
				{
				send_msg(fd,"AUTH_ERROR");
				return;
				}
			crypted = crypt(password, "HI");
			if (strcmp(crypted, daemon_password))
				{
				send_msg(fd,"AUTH_ERROR");
				return;
				}
			nexxusp->authenticated = 1;
			send_msg(fd, "AUTH_OK");
			return;
			}
		/* Should probably say something here */
		printf("[PROXY] Unable to process packet as connection not authed\n");
		return;
		}
	if (!strcmp(type, "PROXY_CONNECT"))
		{
		char						*addr;
		struct in_addr	daemon_addr;

		if (nexxusp->out_sock != -1)
			{
			send_msg(fd, "PROXY_CONNECT_FAIL:ALREADY_CONNECTED");
			return;
			}
		memset(&daemon_addr, 0, sizeof(daemon_addr));
		if (!(addr = strtok(NULL,":")))
			{
			send_msg(fd, "PROXY_CONNECT_FAIL:BAD_ADDRESS");
			return;
			}
		if (!inet_aton(addr, &daemon_addr))
			{
			send_msg(fd, "PROXY_CONNECT_FAIL:BAD_ADDRESS");
			return;
			}
		memcpy(&nexxusp->out_addr, &daemon_addr, sizeof(daemon_addr));
		do_connect(nexxusp);
		return;
		}
	else
		{
		/* Forward request off */
		send_msg(nexxusp->out_sock, saved_msg);
		}
}

static int do_connect(NEXXUS	*nexxusp)
{
	struct sockaddr_in	sock;
	int									optval = 1, rc = 0, fd, optlen;
	unsigned long				sock_opts;
	fd_set							write_fds;
	struct timeval			to;

	fd = nexxusp->in_sock;
	if ((nexxusp->out_sock = socket(AF_INET, SOCK_STREAM, 0)) <0)
		{
		send_msg(fd, "PROXY_CONNECT_FAIL:FAILED_SYSCALL:SOCKET");
		return(-1);
		}
	if (setsockopt(nexxusp->out_sock,
								 SOL_SOCKET,
								 SO_REUSEADDR,
								 &optval,
								 sizeof(optval)) <0)
		{
		send_msg(fd, "PROXY_CONNECT_FAIL:FAILED_SYSCALL:SETSOCKOPT");
		return(-1);
		}
  if ((int)(sock_opts =fcntl(nexxusp->out_sock, F_GETFL))<0)
    {
		send_msg(fd, "PROXY_CONNECT_FAIL:FAILED_SYSCALL:FCNTL");
    return(-1);
    }
  if (fcntl(nexxusp->out_sock, F_SETFL, O_NONBLOCK)<0)
    {
		send_msg(fd, "PROXY_CONNECT_FAIL:FAILED_SYSCALL:FCNTL");
    return(-1);
    }
  memset(&sock, 0, sizeof(sock));
  sock.sin_family = AF_INET;
	memcpy(&sock.sin_addr, &nexxusp->out_addr, sizeof(struct in_addr));
  sock.sin_port = htons(NETWORK_PORT);

	if (debug)
		printf("[PROXY] Connecting to %s\n",inet_ntoa(sock.sin_addr));
  if (connect(nexxusp->out_sock,
              (struct sockaddr *) &sock,
              sizeof(struct sockaddr)) <0)
    {
    if (errno != EINPROGRESS)
      {
			send_msg(fd, "PROXY_CONNECT_FAIL:FAILED_SYSCALL:CONNECT");
			return(-1);
      }
    }

  FD_ZERO(&write_fds);
  FD_SET(nexxusp->out_sock, &write_fds);
  to.tv_usec=0;
  to.tv_sec=5;
  if ((rc = select((nexxusp->out_sock+1), NULL, &write_fds, NULL, &to)) < 0)
    {
		send_msg(fd, "PROXY_CONNECT_FAIL:FAILED_SYSCALL:SELECT");
    return(-1);
    }
  if (rc == 0)
    {
		if (debug)
			printf("[PROXY] connect to %s timed out\n",inet_ntoa(sock.sin_addr));
		send_msg(fd, "PROXY_CONNECT_FAIL:TIMED_OUT");
    return(-1);
    }
  if (!FD_ISSET(nexxusp->out_sock, &write_fds))
    {
		send_msg(fd, "PROXY_CONNECT_FAIL:INTERNAL_ERROR");
    return(-1);
    }
  optlen = sizeof(optval);
  if (getsockopt(nexxusp->out_sock,
                 SOL_SOCKET,
                 SO_ERROR,
                 &optval,
                 &optlen) <0)
    {
		send_msg(fd, "PROXY_CONNECT_FAIL:FAILED_SYSCALL:GETSOCKOPT");
    return(-1);
    }
	if (debug)
		printf("[PROXY] Connection established (errno=%d)\n",optval);
  /* Set the socket to be blocking again */
  if (fcntl(nexxusp->out_sock, F_SETFL, sock_opts)<0)
    {
		send_msg(fd, "PROXY_CONNECT_FAIL:FAILED_SYSCALL:FCNTL");
    return(-1);
    }
#ifdef OPENSSL
  if (!(nexxusp->out_ssl = SSL_new(client_ctx)))
		{
		send_msg(fd, "PROXY_CONNECT_FAIL:FAILED_SYSCALL:SSL_NEW");
    return(-1);
		}
  SSL_set_fd(nexxusp->out_ssl, nexxusp->out_sock);
  if ((rc = SSL_connect(nexxusp->out_ssl))<0)
    {
		send_msg(fd, "PROXY_CONNECT_FAIL:ENCRYPTION_FAILED");
    return(-1);
    }

  if (!(nexxusp->cert=SSL_get_peer_certificate(nexxusp->out_ssl)))
    {
		send_msg(fd, "PROXY_CONNECT_FAIL:ENCRYPTION_FAILED");
		SSL_shutdown(nexxusp->in_ssl);
		close(nexxusp->in_sock);
		close(nexxusp->out_sock);
    return(-1);
    }
  X509_free(nexxusp->cert);
#endif
	send_msg(fd, "PROXY_CONNECT_OK");
  return(0);
}


static void send_msg(int fd, char *msg)
{
	__uint32_t	len;
	__uint32_t	netlen;
#ifdef OPENSSL
	int					rc;
	SSL					*ssl;
#endif

	if (debug)
		printf("[VACM_SYS_STAT_PROXY] Sending msg <%s>\n",msg);
	len = strlen(msg)+1;
	netlen = htonl(len);

#ifdef OPENSSL
	ssl = lookup_nexxus_ssl_by_fd(fd);
  rc = SSL_write(ssl, (char *) &netlen, sizeof(netlen));
	if ((!rc) || (rc <0))
		{
		ERR_print_errors_fp(stderr);
		return;
		}
  rc = SSL_write(ssl, msg, (int) len);
	if ((!rc) || (rc <0))
		{
		ERR_print_errors_fp(stderr);
		return;
		}
#else
	if (write(fd, (char *) &netlen, sizeof(netlen))<0)
		{
		perror("write length");
		return;
		}
	if (write(fd, (char *) msg, len)<0)
		{
		perror("write data");
		return;
		}
#endif
}

#ifdef OPENSSL
static SSL *lookup_nexxus_ssl_by_fd(int fd)
{
	int	I_c1;

	for (I_c1=0; I_c1<255;I_c1++)
		if (nexxus[I_c1].in_sock == fd)
			return(nexxus[I_c1].in_ssl);
  return(NULL);
}
#endif

static int set_password()
{
	FILE						*Tfilep_ptr;
	struct termios	term;
	char						passwd1[255];
	char						passwd2[255];
	char						*crypted;
	char						filename[255];

	if (passwdpath[0]=='\0')
		strcpy(filename,"/etc/vacm_sys_stat_proxy.passwd");
	else
		strcpy(filename,passwdpath);

	unlink(filename);
	if (!(Tfilep_ptr = fopen(filename,"w")))
		{
		printf("Unable to create %s (%m)\n",filename);
		return(0);
		}

	tcgetattr(fileno(stdin), &term);
	term.c_lflag &= ~ECHO;
	tcsetattr(fileno(stdin), TCSANOW, &term);

	fprintf(stdout,"Enter new password -=>");
	fflush(stdout);
	fgets(passwd1,sizeof(passwd1),stdin);
	passwd1[(strlen(passwd1)-1)]=0;
	fprintf(stdout,"\nRe-enter new password -=>");
	fgets(passwd2,sizeof(passwd2),stdin);
	passwd2[(strlen(passwd2)-1)]=0;
	term.c_lflag |= ECHO;
	tcsetattr(fileno(stdin), TCSANOW, &term);
	printf("\n");
	if (strcmp(passwd1, passwd2))
		{
		printf("Passwords do not match\n");
		fclose(Tfilep_ptr);
		return(0);
		}
	crypted = crypt(passwd1, "HI");
	memset(passwd1,0,sizeof(passwd1));
	memset(passwd2,0,sizeof(passwd2));
	fprintf(Tfilep_ptr,"%s\n",crypted);
	strcpy(daemon_password, crypted);
	fclose(Tfilep_ptr);
	return(1);
}

static int get_password()
{
	FILE	*Tfilep_ptr;
	char	filename[255];

	if (passwdpath[0]=='\0')
		strcpy(filename,"/etc/vacm_sys_stat_proxy.passwd");
	else
		strcpy(filename,passwdpath);

	if (!(Tfilep_ptr = fopen(filename,"r")))
		{
		printf("Unable to open %s (%m)\n",filename);
		return(0);
		}
	if (!fgets(daemon_password, sizeof(daemon_password), Tfilep_ptr))
		{
		fclose(Tfilep_ptr);
		return(0);
		}
	daemon_password[(strlen(daemon_password)-1)]=0;
	fclose(Tfilep_ptr);
	return(1);
}

static void become_daemon()
{ 
  int i;
  int fd;
  
  chdir("/");
  if (fork())
    _exit(0);
  setsid(); 
  close(0); 
  close(1); 
  close(2); 
  for (i=0; i < 3; i++)
    {
    fd = open("/dev/null", O_RDWR, 0);
    if (fd < 0) 
      fd = open("/dev/null", O_WRONLY,0);
    if (fd < 0)
      {
      fprintf(stderr,"Unable to detach\n");
      _exit(-1);
      }
    if (fd != i)
      {
      fprintf(stderr,"Unable to detach\n");
      _exit(-1);
      }
    } 
}   
