
/*
 * This program is original work by San Mehat <nettwerk@valinux.com> .
 * This program is distributed under the GNU General Public License (GPL)
 * as outlined in the COPYING file.
 *
 * Copyright (C) 1999, San Mehat, and VA Linux Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 */

#include <config.h>

#include <stdio.h>
#include <sys/socket.h>
#include <signal.h>
#include <netinet/in.h>
#include <sys/un.h>
#include <stdlib.h>
#include <sys/time.h>
#include <errno.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fnmatch.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <nexxus.h>
#include <prototypes.h>
#include <ipc_connection.h>
#include <nexxus_nodes.h>
NODEGROUP	*nodegroup_head = NULL;
NODEGROUP	*nodegroup_tail = NULL;
/*
 *------------------------------------------------------------------------
 * Functions for groups
 *------------------------------------------------------------------------
 */

/*
 * Create a new nodegroup and link it in
 */
NODEGROUP *nodegroup_create(char *group_name)
{
	NODEGROUP	*new;
	char			tmp_str[255];

	sprintf(tmp_str,"nodegroup <%s> structure",group_name);
	new = (NODEGROUP *) nettmcs_alloc(sizeof(NODEGROUP),tmp_str);
	memset(new, 0, sizeof(NODEGROUP));

  if (!nodegroup_tail)
    nodegroup_tail = nodegroup_head = new;
  else
    {
    nodegroup_tail->next = new;
    new->prev = nodegroup_tail;
    }
  nodegroup_tail = new;
  strcpy(new->group_name, group_name);
  return(new);
}

/*
 * Destroy a nodegroup and unlink it
 * Move any nodes under this group to the default group
 */
void nodegroup_destroy(NODEGROUP *group)
{
  if (group == nodegroup_head)
    {
    nodegroup_head = nodegroup_head->next;
    if (nodegroup_head)
      nodegroup_head->prev = NULL;
    else
      nodegroup_tail = NULL;
    }
  else if (group == nodegroup_tail)
    {
    nodegroup_tail = nodegroup_tail->prev;
    if (nodegroup_tail)
      nodegroup_tail->next = NULL;
    else
      nodegroup_head = NULL;
    }
  else
    {
    group->next->prev = group->prev;
    group->prev->next = group->next;
    }
}

/*
 * Look up a nodegroup by name
 */
NODEGROUP *nodegroup_lookup_by_name(char *group_name)
{
	NODEGROUP	*scan;

	scan = nodegroup_head;
	while(scan)
		{
		if (!strcasecmp(group_name, scan->group_name))
			return(scan);
		scan = scan->next;
		}
	return(NULL);
}

/*
 * Look up a nodegroup by a node
 */
NODEGROUP *nodegroup_lookup_by_node(NODE *node)
{
	NODEGROUP				*grp_scan;
	NODE	*node_scan;

	grp_scan = nodegroup_head;
	while(grp_scan)
		{
		node_scan = grp_scan->nodelist;
		while(node_scan)
			{
			if (node_scan == node)
				return(grp_scan);

			node_scan = node_scan->next;
			}
		grp_scan = grp_scan->next;
		}
	return(NULL);
}

/*
 * Add a node to a nodegroup
 */
void nodegroup_add_node(NODEGROUP *group, NODE *node)
{
	NODE	*scan;

	if (!group->nodelist)
		group->nodelist = node;
	else
		{
		for (scan = group->nodelist; scan -> next; scan = scan->next);
		scan->next = node;
		node->prev = scan;
		}
}

/*
 * Remove a node from a nodegroup
 */
void nodegroup_remove_node(NODEGROUP *group, NODE *node)
{
	NODE	*node_scan;

	if (node == group->nodelist)
		group->nodelist = group->nodelist->next;
	else
		{
		node_scan = group->nodelist;
		while(node_scan->next != node)
			node_scan = node_scan->next;
		node_scan->next = node->next;
		if (node->next)
			node->next->prev = node_scan;
		}
}

/*
 *------------------------------------------------------------------------
 * Functions for nodes
 *------------------------------------------------------------------------
 */

/*
 * Create a new node structure
 */
NODE *nodelist_entry_create(char *node_id)
{
  NODE  *new;
	char						tmp_str[255];

	sprintf(tmp_str,"nodelist entry <%s>",node_id);
  if (!(new = (NODE *) nettmcs_alloc (sizeof(NODE),
                                               	tmp_str)))
    {
    nexxus_log_printf(NULL,"Unable to allocate memory for Nodelist Entry\n");
    return(NULL);
    }
  memset(new, 0, sizeof(NODE));

  strcpy(new->node_id, node_id);
  return(new);
}

/*
 * Lookup a node in a nodegroup by node name
 * If NULL is passed in for group, all groups will be checked
 */
NODE *nodegroup_node_lookup(NODEGROUP *group, char *node_name)
{
	NODE	*node_scan;
	NODEGROUP				*group_scan;

	if (group)
		{
		node_scan = group->nodelist;
		while(node_scan)
			{
			if (!strcasecmp(node_scan->node_id, node_name))
				return(node_scan);
			node_scan = node_scan->next;
			}
		}
	else
		{
		group_scan = nodegroup_head;
		while(group_scan)
			{
			node_scan = group_scan->nodelist;
			while(node_scan)
				{
				if (!strcasecmp(node_scan->node_id, node_name))
					return(node_scan);
				node_scan = node_scan->next;
				}
			group_scan = group_scan->next;
			}
		}
	return(0);
}

/*
 * Rename a node structure
 * Returns -1 if target name is in use
 * Returns -2 if node is not found (or user is not in the same group)
 */
int nodelist_entry_rename(NEXXUS_USER	*user, char *old_name, char *new_name)
{
  NODE 			 *node;
  NODEGROUP	*group;

	if (!(node =nodegroup_node_lookup(NULL, old_name)))
		return(-2); /* node not found */
	if (!(group =nodegroup_lookup_by_node(node)))
		{
		nexxus_log_printf(NULL,"INTERNAL ERROR while renaming\n");
		nexxus_log_printf(NULL,"Node %s does not have a group??\n",old_name);
		return(-2);
		}
	if (!user_group_user_is_member(user, group->group_name))
		return(-2); /* naughty naughty */
	/* Check if new name exists */
	if (nodegroup_node_lookup(NULL,new_name))
		return(-1); /* target in use */
	/* rename it */
	strcpy(node->node_id, new_name);
	return(0);
}

/*
 * Searches through the nodelist looking for nodes that match the glob
 * specified. To avoid having static data structures, pointers to the 
 * previous group and node are provided by the caller, and passed in
 * upon successive calls. To initialize the search, the last_gr and
 * last_n parameters must point to a NULL pointer. Successive calls
 * will return the next matching nodes until no more nodes match, at
 * which point NULL is returned
 */
NODE *node_lookup_by_glob(char *glob, NODEGROUP **last_gr, NODE **last_n)
{
	if (*last_gr == NULL)
		{
		*last_gr = nodegroup_head;
		*last_n =  ((NODEGROUP *)(*last_gr))->nodelist;
		}
	else
		*last_n = ((NODE *)(*last_n))->next;
	/* Scan through list startingat last_gr and last_n looking for glob matches */
	while(*last_gr)
		{
		while(*last_n)
			{
			if (!(fnmatch(glob, ((NODE *)(*last_n))->node_id, 0)))
				return(*last_n);
			*last_n = ((NODE *)(*last_n))->next;
			}
		*last_gr = ((NODEGROUP *)(*last_gr))->next;
		if (*last_gr)
			*last_n =  ((NODEGROUP *)(*last_gr))->nodelist;
		}
	return(NULL);
}

/*
 * Delete a node structure (if the user is in the same group)
 */
int nodelist_entry_delete(NEXXUS_USER *user, char *node_id)
{
	NODEGROUP				*group;
	NODE	*node;

	if (!(node =nodegroup_node_lookup(NULL, node_id)))
		return(-1); /* node not found */
	if (!(group =nodegroup_lookup_by_node(node)))
		{
		nexxus_log_printf(NULL,"INTERNAL ERROR while deleting\n");
		nexxus_log_printf(NULL,"Node %s does not have a group??\n",node_id);
		return(-1);
		}
	if (!user_group_user_is_member(user, group->group_name))
		return(-2); /* naughty naughty */
	nodegroup_remove_node(group, node);
	nettmcs_free(node);
	return(0);
}

void verify_groups()
{
	NODEGROUP	*default_group;

	if (nodegroup_head)
		return;
	nexxus_log_printf(NULL,
												"No groups. Creating 'default' group\n");
	if (!(default_group = nodegroup_create("default")))
		nexxus_log_printf(NULL,"Unable to create default group\n");
	configuration_save();
}

